@@ -22,10 +22,10 @@
 
 #include <adbc.h>
 
+#include "common/utils.h"
 #include "connection.h"
 #include "database.h"
 #include "statement.h"
-#include "util.h"
 
 using adbcpq::PostgresConnection;
 using adbcpq::PostgresDatabase;
@@ -51,7 +51,6 @@ using adbcpq::PostgresStatement;
 // AdbcDatabase
 
 namespace {
-using adbcpq::SetError;
 AdbcStatusCode PostgresDatabaseInit(struct AdbcDatabase* database,
                                     struct AdbcError* error) {
   if (!database || !database->private_data) return ADBC_STATUS_INVALID_STATE;
@@ -62,11 +61,11 @@ AdbcStatusCode PostgresDatabaseInit(struct AdbcDatabase* database,
 AdbcStatusCode PostgresDatabaseNew(struct AdbcDatabase* database,
                                    struct AdbcError* error) {
   if (!database) {
-    SetError(error, "database must not be null");
+    SetError(error, "%s", "[libpq] database must not be null");
     return ADBC_STATUS_INVALID_STATE;
   }
   if (database->private_data) {
-    SetError(error, "database is already initialized");
+    SetError(error, "%s", "[libpq] database is already initialized");
     return ADBC_STATUS_INVALID_STATE;
   }
   auto impl = std::make_shared<PostgresDatabase>();
@@ -126,14 +125,21 @@ AdbcStatusCode PostgresConnectionGetInfo(struct AdbcConnection* connection,
                                          uint32_t* info_codes, size_t info_codes_length,
                                          struct ArrowArrayStream* stream,
                                          struct AdbcError* error) {
-  return ADBC_STATUS_NOT_IMPLEMENTED;
+  if (!connection->private_data) return ADBC_STATUS_INVALID_STATE;
+  auto ptr =
+      reinterpret_cast<std::shared_ptr<PostgresConnection>*>(connection->private_data);
+  return (*ptr)->GetInfo(connection, info_codes, info_codes_length, stream, error);
 }
 
 AdbcStatusCode PostgresConnectionGetObjects(
     struct AdbcConnection* connection, int depth, const char* catalog,
     const char* db_schema, const char* table_name, const char** table_types,
     const char* column_name, struct ArrowArrayStream* stream, struct AdbcError* error) {
-  return ADBC_STATUS_NOT_IMPLEMENTED;
+  if (!connection->private_data) return ADBC_STATUS_INVALID_STATE;
+  auto ptr =
+      reinterpret_cast<std::shared_ptr<PostgresConnection>*>(connection->private_data);
+  return (*ptr)->GetObjects(connection, depth, catalog, db_schema, table_name,
+                            table_types, column_name, stream, error);
 }
 
 AdbcStatusCode PostgresConnectionGetTableSchema(
@@ -148,7 +154,10 @@ AdbcStatusCode PostgresConnectionGetTableSchema(
 AdbcStatusCode PostgresConnectionGetTableTypes(struct AdbcConnection* connection,
                                                struct ArrowArrayStream* stream,
                                                struct AdbcError* error) {
-  return ADBC_STATUS_NOT_IMPLEMENTED;
+  if (!connection->private_data) return ADBC_STATUS_INVALID_STATE;
+  auto ptr =
+      reinterpret_cast<std::shared_ptr<PostgresConnection>*>(connection->private_data);
+  return (*ptr)->GetTableTypes(connection, stream, error);
 }
 
 AdbcStatusCode PostgresConnectionInit(struct AdbcConnection* connection,
@@ -307,8 +316,6 @@ AdbcStatusCode PostgresStatementExecutePartitions(struct AdbcStatement* statemen
                                                   int64_t* rows_affected,
                                                   struct AdbcError* error) {
   if (!statement->private_data) return ADBC_STATUS_INVALID_STATE;
-  auto* ptr =
-      reinterpret_cast<std::shared_ptr<PostgresStatement>*>(statement->private_data);
   return ADBC_STATUS_NOT_IMPLEMENTED;
 }
 
