@@ -45,7 +45,36 @@ class SqliteQuirks : public adbc_validation::DriverQuirks {
 
   std::string BindParameter(int index) const override { return "?"; }
 
+  ArrowType IngestSelectRoundTripType(ArrowType ingest_type) const override {
+    switch (ingest_type) {
+      case NANOARROW_TYPE_INT8:
+      case NANOARROW_TYPE_INT16:
+      case NANOARROW_TYPE_INT32:
+      case NANOARROW_TYPE_INT64:
+      case NANOARROW_TYPE_UINT8:
+      case NANOARROW_TYPE_UINT16:
+      case NANOARROW_TYPE_UINT32:
+      case NANOARROW_TYPE_UINT64:
+        return NANOARROW_TYPE_INT64;
+      case NANOARROW_TYPE_FLOAT:
+      case NANOARROW_TYPE_DOUBLE:
+        return NANOARROW_TYPE_DOUBLE;
+      default:
+        return ingest_type;
+    }
+  }
+
+  std::optional<std::string> PrimaryKeyTableDdl(std::string_view name) const override {
+    std::string ddl = "CREATE TABLE ";
+    ddl += name;
+    ddl += " (id INTEGER PRIMARY KEY)";
+    return ddl;
+  }
+
   bool supports_concurrent_statements() const override { return true; }
+
+  std::string catalog() const override { return "main"; }
+  std::string db_schema() const override { return ""; }
 };
 
 class SqliteDatabaseTest : public ::testing::Test, public adbc_validation::DatabaseTest {
@@ -71,6 +100,68 @@ class SqliteConnectionTest : public ::testing::Test,
 };
 ADBCV_TEST_CONNECTION(SqliteConnectionTest)
 
+TEST_F(SqliteConnectionTest, GetInfoMetadata) {
+  ASSERT_THAT(AdbcConnectionNew(&connection, &error),
+              adbc_validation::IsOkStatus(&error));
+  ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error),
+              adbc_validation::IsOkStatus(&error));
+
+  adbc_validation::StreamReader reader;
+  std::vector<uint32_t> info = {
+      ADBC_INFO_DRIVER_NAME,
+      ADBC_INFO_DRIVER_VERSION,
+      ADBC_INFO_VENDOR_NAME,
+      ADBC_INFO_VENDOR_VERSION,
+  };
+  ASSERT_THAT(AdbcConnectionGetInfo(&connection, info.data(), info.size(),
+                                    &reader.stream.value, &error),
+              adbc_validation::IsOkStatus(&error));
+  ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
+
+  std::vector<uint32_t> seen;
+  while (true) {
+    ASSERT_NO_FATAL_FAILURE(reader.Next());
+    if (!reader.array->release) break;
+
+    for (int64_t row = 0; row < reader.array->length; row++) {
+      ASSERT_FALSE(ArrowArrayViewIsNull(reader.array_view->children[0], row));
+      const uint32_t code =
+          reader.array_view->children[0]->buffer_views[1].data.as_uint32[row];
+      seen.push_back(code);
+
+      int str_child_index = 0;
+      struct ArrowArrayView* str_child =
+          reader.array_view->children[1]->children[str_child_index];
+      switch (code) {
+        case ADBC_INFO_DRIVER_NAME: {
+          ArrowStringView val = ArrowArrayViewGetStringUnsafe(str_child, 0);
+          EXPECT_EQ("ADBC SQLite Driver", std::string(val.data, val.size_bytes));
+          break;
+        }
+        case ADBC_INFO_DRIVER_VERSION: {
+          ArrowStringView val = ArrowArrayViewGetStringUnsafe(str_child, 1);
+          EXPECT_EQ("(unknown)", std::string(val.data, val.size_bytes));
+          break;
+        }
+        case ADBC_INFO_VENDOR_NAME: {
+          ArrowStringView val = ArrowArrayViewGetStringUnsafe(str_child, 2);
+          EXPECT_EQ("SQLite", std::string(val.data, val.size_bytes));
+          break;
+        }
+        case ADBC_INFO_VENDOR_VERSION: {
+          ArrowStringView val = ArrowArrayViewGetStringUnsafe(str_child, 3);
+          EXPECT_THAT(std::string(val.data, val.size_bytes),
+                      ::testing::MatchesRegex("3\\..*"));
+        }
+        default:
+          // Ignored
+          break;
+      }
+    }
+  }
+  ASSERT_THAT(seen, ::testing::UnorderedElementsAreArray(info));
+}
+
 class SqliteStatementTest : public ::testing::Test,
                             public adbc_validation::StatementTest {
  public:
@@ -78,6 +169,9 @@ class SqliteStatementTest : public ::testing::Test,
   void SetUp() override { ASSERT_NO_FATAL_FAILURE(SetUpTest()); }
   void TearDown() override { ASSERT_NO_FATAL_FAILURE(TearDownTest()); }
 
+  void TestSqlIngestUInt64() { GTEST_SKIP() << "Cannot ingest UINT64 (out of range)"; }
+  void TestSqlIngestBinary() { GTEST_SKIP() << "Cannot ingest BINARY (not implemented)"; }
+
  protected:
   SqliteQuirks quirks_;
 };
@@ -159,7 +253,7 @@ class SqliteReaderTest : public ::testing::Test {
 TEST_F(SqliteReaderTest, IntsNulls) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(ExecSelect("(NULL), (1), (NULL), (-1)", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<int64_t>(reader.array_view->children[0],
@@ -170,7 +264,7 @@ TEST_F(SqliteReaderTest, FloatsNulls) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect("(NULL), (1.0), (NULL), (-1.0), (0.0)", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<double>(
@@ -181,7 +275,7 @@ TEST_F(SqliteReaderTest, IntsFloatsNulls) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect("(NULL), (1), (NULL), (-1.0), (0)", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<double>(
@@ -192,7 +286,7 @@ TEST_F(SqliteReaderTest, IntsNullsStrsNullsInts) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(ExecSelect(
       R"((NULL), (1), (NULL), (-1), ("foo"), (NULL), (""), (24))", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(
@@ -205,7 +299,7 @@ TEST_F(SqliteReaderTest, IntExtremes) {
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((NULL), (9223372036854775807), (NULL), (-9223372036854775808))",
                  kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
@@ -219,7 +313,7 @@ TEST_F(SqliteReaderTest, IntExtremesStrs) {
   ASSERT_NO_FATAL_FAILURE(ExecSelect(
       R"((NULL), (9223372036854775807), (-9223372036854775808), (""), (9223372036854775807), (-9223372036854775808))",
       kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(reader.array_view->children[0],
@@ -237,7 +331,7 @@ TEST_F(SqliteReaderTest, FloatExtremes) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((NULL), (9e999), (NULL), (-9e999))", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(CompareArray<double>(
@@ -253,7 +347,7 @@ TEST_F(SqliteReaderTest, IntsFloatsStrs) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((1), (1.0), (""), (9e999), (-9e999))", kInferRows, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
@@ -265,7 +359,7 @@ TEST_F(SqliteReaderTest, InferIntReadInt) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((1), (NULL), (2), (NULL))", /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<int64_t>(reader.array_view->children[0], {1, std::nullopt}));
@@ -280,7 +374,7 @@ TEST_F(SqliteReaderTest, InferIntRejectFloat) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((1), (NULL), (2E0), (NULL))", /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<int64_t>(reader.array_view->children[0], {1, std::nullopt}));
@@ -295,7 +389,7 @@ TEST_F(SqliteReaderTest, InferIntRejectStr) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((1), (NULL), (""), (NULL))", /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<int64_t>(reader.array_view->children[0], {1, std::nullopt}));
@@ -311,7 +405,7 @@ TEST_F(SqliteReaderTest, InferFloatReadIntFloat) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(
       ExecSelect(R"((1E0), (NULL), (2E0), (3), (NULL))", /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<double>(reader.array_view->children[0], {1.0, std::nullopt}));
@@ -329,7 +423,7 @@ TEST_F(SqliteReaderTest, InferFloatRejectStr) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(ExecSelect(R"((1E0), (NULL), (2E0), (3), (""), (NULL))",
                                      /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_DOUBLE, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<double>(reader.array_view->children[0], {1.0, std::nullopt}));
@@ -348,7 +442,7 @@ TEST_F(SqliteReaderTest, InferStrReadAll) {
   adbc_validation::StreamReader reader;
   ASSERT_NO_FATAL_FAILURE(ExecSelect(R"((""), (NULL), (2), (3E0), ("foo"), (NULL))",
                                      /*infer_rows=*/2, &reader));
-  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_STRING, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<std::string>(reader.array_view->children[0], {"", std::nullopt}));
@@ -378,7 +472,7 @@ TEST_F(SqliteReaderTest, InferOneParam) {
   ASSERT_NO_FATAL_FAILURE(Exec("SELECT ?", /*infer_rows=*/2, &reader));
 
   ASSERT_EQ(1, reader.schema->n_children);
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<int64_t>(reader.array_view->children[0], {std::nullopt, 2}));
@@ -411,7 +505,7 @@ TEST_F(SqliteReaderTest, InferOneParamStream) {
   ASSERT_NO_FATAL_FAILURE(Exec("SELECT ?", /*infer_rows=*/3, &reader));
 
   ASSERT_EQ(1, reader.schema->n_children);
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
       CompareArray<int64_t>(reader.array_view->children[0], {std::nullopt, 1, 2}));
@@ -441,7 +535,7 @@ TEST_F(SqliteReaderTest, InferTypedParams) {
   ASSERT_NO_FATAL_FAILURE(
       Exec("SELECT value FROM foo WHERE idx = ?", /*infer_rows=*/2, &reader));
   ASSERT_EQ(1, reader.schema->n_children);
-  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].data_type);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
   ASSERT_NO_FATAL_FAILURE(
@@ -452,6 +546,47 @@ TEST_F(SqliteReaderTest, InferTypedParams) {
                   "[SQLite] Type mismatch in column 0: expected INT64 but got DOUBLE"));
 }
 
+TEST_F(SqliteReaderTest, MultiValueParams) {
+  // Regression test for apache/arrow-adbc#734
+  adbc_validation::StreamReader reader;
+  Handle<struct ArrowSchema> schema;
+  Handle<struct ArrowArray> batch;
+
+  ASSERT_NO_FATAL_FAILURE(Exec("CREATE TABLE foo (col)"));
+  ASSERT_NO_FATAL_FAILURE(
+      Exec("INSERT INTO foo VALUES (1), (2), (2), (3), (3), (3), (4), (4), (4), (4)"));
+
+  ASSERT_THAT(adbc_validation::MakeSchema(&schema.value, {{"", NANOARROW_TYPE_INT64}}),
+              IsOkErrno());
+  ASSERT_THAT(adbc_validation::MakeBatch<int64_t>(&schema.value, &batch.value,
+                                                  /*error=*/nullptr, {4, 1, 3, 2}),
+              IsOkErrno());
+
+  ASSERT_NO_FATAL_FAILURE(Bind(&batch.value, &schema.value));
+  ASSERT_NO_FATAL_FAILURE(
+      Exec("SELECT col FROM foo WHERE col = ?", /*infer_rows=*/3, &reader));
+  ASSERT_EQ(1, reader.schema->n_children);
+  ASSERT_EQ(NANOARROW_TYPE_INT64, reader.fields[0].type);
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NO_FATAL_FAILURE(
+      CompareArray<int64_t>(reader.array_view->children[0], {4, 4, 4}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NO_FATAL_FAILURE(
+      CompareArray<int64_t>(reader.array_view->children[0], {4, 1, 3}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NO_FATAL_FAILURE(
+      CompareArray<int64_t>(reader.array_view->children[0], {3, 3, 2}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NO_FATAL_FAILURE(CompareArray<int64_t>(reader.array_view->children[0], {2}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_EQ(nullptr, reader.array->release);
+}
+
 template <typename CType>
 class SqliteNumericParamTest : public SqliteReaderTest,
                                public ::testing::WithParamInterface<ArrowType> {
@@ -472,7 +607,7 @@ class SqliteNumericParamTest : public SqliteReaderTest,
     ASSERT_NO_FATAL_FAILURE(Exec("SELECT ?", /*infer_rows=*/2, &reader));
 
     ASSERT_EQ(1, reader.schema->n_children);
-    ASSERT_EQ(expected_type, reader.fields[0].data_type);
+    ASSERT_EQ(expected_type, reader.fields[0].type);
     ASSERT_NO_FATAL_FAILURE(reader.Next());
     ASSERT_NO_FATAL_FAILURE(
         CompareArray<CType>(reader.array_view->children[0], {std::nullopt, 0}));
