@@ -17,9 +17,10 @@
 
 #include "adbc_validation.h"
 
+#include <algorithm>
 #include <cerrno>
 #include <cstring>
-#include <iostream>
+#include <limits>
 #include <optional>
 #include <string>
 #include <string_view>
@@ -51,6 +52,15 @@ namespace {
       return adbc_status;                                           \
     }                                                               \
   } while (false)
+
+/// case insensitive string compare
+bool iequals(std::string_view s1, std::string_view s2) {
+  return std::equal(s1.begin(), s1.end(), s2.begin(), s2.end(),
+                    [](unsigned char a, unsigned char b) {
+                      return std::tolower(a) == std::tolower(b);
+                    });
+}
+
 }  // namespace
 
 //------------------------------------------------------------
@@ -272,6 +282,25 @@ void ConnectionTest::TestMetadataGetInfo() {
                         {"string_list", NANOARROW_TYPE_LIST, NULLABLE},
                         {"int32_to_int32_list_map", NANOARROW_TYPE_MAP, NULLABLE},
                     }));
+  ASSERT_NO_FATAL_FAILURE(CompareSchema(reader.schema->children[1]->children[4],
+                                        {
+                                            {"item", NANOARROW_TYPE_STRING, NULLABLE},
+                                        }));
+  ASSERT_NO_FATAL_FAILURE(CompareSchema(reader.schema->children[1]->children[5],
+                                        {
+                                            {"entries", NANOARROW_TYPE_STRUCT, NOT_NULL},
+                                        }));
+  ASSERT_NO_FATAL_FAILURE(
+      CompareSchema(reader.schema->children[1]->children[5]->children[0],
+                    {
+                        {"key", NANOARROW_TYPE_INT32, NOT_NULL},
+                        {"value", NANOARROW_TYPE_LIST, NULLABLE},
+                    }));
+  ASSERT_NO_FATAL_FAILURE(
+      CompareSchema(reader.schema->children[1]->children[5]->children[0]->children[1],
+                    {
+                        {"item", NANOARROW_TYPE_INT32, NULLABLE},
+                    }));
 
   std::vector<uint32_t> seen;
   while (true) {
@@ -482,12 +511,18 @@ void ConnectionTest::TestMetadataGetObjectsDbSchemas() {
         ASSERT_FALSE(ArrowArrayViewIsNull(catalog_db_schemas_list, row))
             << "Row " << row << " should have non-null catalog_db_schemas";
 
+        ArrowStringView catalog_name =
+            ArrowArrayViewGetStringUnsafe(reader.array_view->children[0], row);
+
         const int64_t start_offset =
-            ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row);
+            ArrowArrayViewListChildOffset(catalog_db_schemas_list, row);
         const int64_t end_offset =
-            ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row + 1);
-        ASSERT_GT(end_offset, start_offset)
-            << "Row " << row << " should have nonempty catalog_db_schemas";
+            ArrowArrayViewListChildOffset(catalog_db_schemas_list, row + 1);
+        ASSERT_GE(end_offset, start_offset)
+            << "Row " << row << " (Catalog "
+            << std::string(catalog_name.data, catalog_name.size_bytes)
+            << ") should have nonempty catalog_db_schemas ";
+        ASSERT_FALSE(ArrowArrayViewIsNull(catalog_db_schemas_list, row));
         for (int64_t list_index = start_offset; list_index < end_offset; list_index++) {
           ASSERT_TRUE(ArrowArrayViewIsNull(db_schema_tables_list, row + list_index))
               << "Row " << row << " should have null db_schema_tables";
@@ -516,9 +551,9 @@ void ConnectionTest::TestMetadataGetObjectsDbSchemas() {
             << "Row " << row << " should have non-null catalog_db_schemas";
 
         const int64_t start_offset =
-            ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row);
+            ArrowArrayViewListChildOffset(catalog_db_schemas_list, row);
         const int64_t end_offset =
-            ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row + 1);
+            ArrowArrayViewListChildOffset(catalog_db_schemas_list, row + 1);
         ASSERT_EQ(start_offset, end_offset);
       }
       ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -571,22 +606,22 @@ void ConnectionTest::TestMetadataGetObjectsTables() {
             << "Row " << row << " should have non-null catalog_db_schemas";
 
         for (int64_t db_schemas_index =
-                 ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row);
+                 ArrowArrayViewListChildOffset(catalog_db_schemas_list, row);
              db_schemas_index <
-             ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row + 1);
+             ArrowArrayViewListChildOffset(catalog_db_schemas_list, row + 1);
              db_schemas_index++) {
-          ASSERT_FALSE(
-              ArrowArrayViewIsNull(db_schema_tables_list, row + db_schemas_index))
+          ASSERT_FALSE(ArrowArrayViewIsNull(db_schema_tables_list, db_schemas_index))
               << "Row " << row << " should have non-null db_schema_tables";
 
-          for (int64_t tables_index = ArrowArrayViewGetOffsetUnsafe(
-                   db_schema_tables_list, row + db_schemas_index);
-               tables_index < ArrowArrayViewGetOffsetUnsafe(db_schema_tables_list,
-                                                            row + db_schemas_index + 1);
+          for (int64_t tables_index =
+                   ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index);
+               tables_index <
+               ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index + 1);
                tables_index++) {
             ArrowStringView table_name = ArrowArrayViewGetStringUnsafe(
                 db_schema_tables->children[0], tables_index);
-            if (std::string_view(table_name.data, table_name.n_bytes) == "bulk_ingest") {
+            if (iequals(std::string(table_name.data, table_name.size_bytes),
+                        "bulk_ingest")) {
               found_expected_table = true;
             }
 
@@ -645,21 +680,22 @@ void ConnectionTest::TestMetadataGetObjectsTablesTypes() {
             << "Row " << row << " should have non-null catalog_db_schemas";
 
         for (int64_t db_schemas_index =
-                 ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row);
+                 ArrowArrayViewListChildOffset(catalog_db_schemas_list, row);
              db_schemas_index <
-             ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row + 1);
+             ArrowArrayViewListChildOffset(catalog_db_schemas_list, row + 1);
              db_schemas_index++) {
           ASSERT_FALSE(ArrowArrayViewIsNull(db_schema_tables_list, db_schemas_index))
               << "Row " << row << " should have non-null db_schema_tables";
 
-          for (int64_t tables_index = ArrowArrayViewGetOffsetUnsafe(
-                   db_schema_tables_list, row + db_schemas_index);
+          for (int64_t tables_index =
+                   ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index);
                tables_index <
-               ArrowArrayViewGetOffsetUnsafe(db_schema_tables_list, db_schemas_index + 1);
+               ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index + 1);
                tables_index++) {
             ArrowStringView table_name = ArrowArrayViewGetStringUnsafe(
                 db_schema_tables->children[0], tables_index);
-            if (std::string_view(table_name.data, table_name.n_bytes) == "bulk_ingest") {
+            if (std::string_view(table_name.data, table_name.size_bytes) ==
+                "bulk_ingest") {
               found_expected_table = true;
             }
 
@@ -741,18 +777,20 @@ void ConnectionTest::TestMetadataGetObjectsColumns() {
             << "Row " << row << " should have non-null catalog_db_schemas";
 
         for (int64_t db_schemas_index =
-                 ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row);
+                 ArrowArrayViewListChildOffset(catalog_db_schemas_list, row);
              db_schemas_index <
-             ArrowArrayViewGetOffsetUnsafe(catalog_db_schemas_list, row + 1);
+             ArrowArrayViewListChildOffset(catalog_db_schemas_list, row + 1);
              db_schemas_index++) {
-          ASSERT_FALSE(
-              ArrowArrayViewIsNull(db_schema_tables_list, row + db_schemas_index))
+          ASSERT_FALSE(ArrowArrayViewIsNull(db_schema_tables_list, db_schemas_index))
               << "Row " << row << " should have non-null db_schema_tables";
 
+          ArrowStringView db_schema_name = ArrowArrayViewGetStringUnsafe(
+              catalog_db_schemas->children[0], db_schemas_index);
+
           for (int64_t tables_index =
-                   ArrowArrayViewGetOffsetUnsafe(db_schema_tables_list, db_schemas_index);
+                   ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index);
                tables_index <
-               ArrowArrayViewGetOffsetUnsafe(db_schema_tables_list, db_schemas_index + 1);
+               ArrowArrayViewListChildOffset(db_schema_tables_list, db_schemas_index + 1);
                tables_index++) {
             ArrowStringView table_name = ArrowArrayViewGetStringUnsafe(
                 db_schema_tables->children[0], tables_index);
@@ -762,17 +800,23 @@ void ConnectionTest::TestMetadataGetObjectsColumns() {
             ASSERT_FALSE(ArrowArrayViewIsNull(table_constraints_list, tables_index))
                 << "Row " << row << " should have non-null table_constraints";
 
-            if (std::string_view(table_name.data, table_name.n_bytes) == "bulk_ingest") {
+            if (iequals(std::string(table_name.data, table_name.size_bytes),
+                        "bulk_ingest") &&
+                iequals(std::string(db_schema_name.data, db_schema_name.size_bytes),
+                        quirks()->db_schema())) {
               found_expected_table = true;
 
               for (int64_t columns_index =
-                       ArrowArrayViewGetOffsetUnsafe(table_columns_list, tables_index);
+                       ArrowArrayViewListChildOffset(table_columns_list, tables_index);
                    columns_index <
-                   ArrowArrayViewGetOffsetUnsafe(table_columns_list, tables_index + 1);
+                   ArrowArrayViewListChildOffset(table_columns_list, tables_index + 1);
                    columns_index++) {
                 ArrowStringView name = ArrowArrayViewGetStringUnsafe(
                     table_columns->children[0], columns_index);
-                column_names.push_back(std::string(name.data, name.n_bytes));
+                std::string temp(name.data, name.size_bytes);
+                std::transform(temp.begin(), temp.end(), temp.begin(),
+                               [](unsigned char c) { return std::tolower(c); });
+                column_names.push_back(std::move(temp));
                 ordinal_positions.push_back(
                     static_cast<int32_t>(ArrowArrayViewGetIntUnsafe(
                         table_columns->children[1], columns_index)));
@@ -794,6 +838,79 @@ void ConnectionTest::TestMetadataGetObjectsConstraints() {
   // TODO: can't be done portably (need to create tables with primary keys and such)
 }
 
+void ConnectionTest::TestMetadataGetObjectsPrimaryKey() {
+  ASSERT_THAT(AdbcConnectionNew(&connection, &error), IsOkStatus(&error));
+  ASSERT_THAT(AdbcConnectionInit(&connection, &database, &error), IsOkStatus(&error));
+
+  if (!quirks()->supports_get_objects()) {
+    GTEST_SKIP();
+  }
+
+  std::optional<std::string> maybe_ddl = quirks()->PrimaryKeyTableDdl("adbc_pkey_test");
+  if (!maybe_ddl.has_value()) {
+    GTEST_SKIP();
+  }
+  std::string ddl = std::move(*maybe_ddl);
+
+  ASSERT_THAT(quirks()->DropTable(&connection, "adbc_pkey_test", &error),
+              IsOkStatus(&error));
+
+  {
+    Handle<AdbcStatement> statement;
+    ASSERT_THAT(AdbcStatementNew(&connection, &statement.value, &error),
+                IsOkStatus(&error));
+    ASSERT_THAT(AdbcStatementSetSqlQuery(&statement.value, ddl.c_str(), &error),
+                IsOkStatus(&error));
+    int64_t rows_affected = 0;
+    ASSERT_THAT(
+        AdbcStatementExecuteQuery(&statement.value, nullptr, &rows_affected, &error),
+        IsOkStatus(&error));
+  }
+
+  adbc_validation::StreamReader reader;
+  ASSERT_THAT(
+      AdbcConnectionGetObjects(&connection, ADBC_OBJECT_DEPTH_ALL, nullptr, nullptr,
+                               nullptr, nullptr, nullptr, &reader.stream.value, &error),
+      IsOkStatus(&error));
+  ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NE(nullptr, reader.array->release);
+  ASSERT_GT(reader.array->length, 0);
+
+  auto get_objects_data = adbc_validation::GetObjectsReader{&reader.array_view.value};
+  ASSERT_NE(*get_objects_data, nullptr)
+      << "could not initialize the AdbcGetObjectsData object";
+
+  struct AdbcGetObjectsTable* table =
+      AdbcGetObjectsDataGetTableByName(*get_objects_data, quirks()->catalog().c_str(),
+                                       quirks()->db_schema().c_str(), "adbc_pkey_test");
+  ASSERT_NE(table, nullptr) << "could not find adbc_pkey_test table";
+
+  ASSERT_EQ(table->n_table_columns, 1);
+  struct AdbcGetObjectsColumn* column = AdbcGetObjectsDataGetColumnByName(
+      *get_objects_data, quirks()->catalog().c_str(), quirks()->db_schema().c_str(),
+      "adbc_pkey_test", "id");
+  ASSERT_NE(column, nullptr) << "could not find id column on adbc_pkey_test table";
+
+  ASSERT_EQ(table->n_table_constraints, 1)
+      << "expected 1 constraint on adbc_pkey_test table, found: "
+      << table->n_table_constraints;
+
+  struct AdbcGetObjectsConstraint* constraint = table->table_constraints[0];
+
+  std::string_view constraint_type(constraint->constraint_type.data,
+                                   constraint->constraint_type.size_bytes);
+  ASSERT_EQ(constraint_type, "PRIMARY KEY");
+  ASSERT_EQ(constraint->n_column_names, 1)
+      << "expected constraint adbc_pkey_test_pkey to be applied to 1 column, found: "
+      << constraint->n_column_names;
+
+  std::string_view constraint_column_name(
+      constraint->constraint_column_names[0].data,
+      constraint->constraint_column_names[0].size_bytes);
+  ASSERT_EQ(constraint_column_name, "id");
+}
+
 //------------------------------------------------------------
 // Tests of AdbcStatement
 
@@ -845,7 +962,9 @@ void StatementTest::TestRelease() {
   ASSERT_EQ(NULL, statement.private_data);
 }
 
-void StatementTest::TestSqlIngestInts() {
+template <typename CType>
+void StatementTest::TestSqlIngestType(ArrowType type,
+                                      const std::vector<std::optional<CType>>& values) {
   if (!quirks()->supports_bulk_ingest()) {
     GTEST_SKIP();
   }
@@ -856,10 +975,9 @@ void StatementTest::TestSqlIngestInts() {
   Handle<struct ArrowSchema> schema;
   Handle<struct ArrowArray> array;
   struct ArrowError na_error;
-  ASSERT_THAT(MakeSchema(&schema.value, {{"int64s", NANOARROW_TYPE_INT64}}), IsOkErrno());
-  ASSERT_THAT(
-      MakeBatch<int64_t>(&schema.value, &array.value, &na_error, {42, -42, std::nullopt}),
-      IsOkErrno());
+  ASSERT_THAT(MakeSchema(&schema.value, {{"col", type}}), IsOkErrno());
+  ASSERT_THAT(MakeBatch<CType>(&schema.value, &array.value, &na_error, values),
+              IsOkErrno());
 
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcStatementSetOption(&statement, ADBC_INGEST_OPTION_TARGET_TABLE,
@@ -871,9 +989,12 @@ void StatementTest::TestSqlIngestInts() {
   int64_t rows_affected = 0;
   ASSERT_THAT(AdbcStatementExecuteQuery(&statement, nullptr, &rows_affected, &error),
               IsOkStatus(&error));
-  ASSERT_THAT(rows_affected, ::testing::AnyOf(::testing::Eq(3), ::testing::Eq(-1)));
+  ASSERT_THAT(rows_affected,
+              ::testing::AnyOf(::testing::Eq(values.size()), ::testing::Eq(-1)));
 
-  ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT * FROM bulk_ingest", &error),
+  ASSERT_THAT(AdbcStatementSetSqlQuery(
+                  &statement,
+                  "SELECT * FROM bulk_ingest ORDER BY \"col\" ASC NULLS FIRST", &error),
               IsOkStatus(&error));
   {
     StreamReader reader;
@@ -881,25 +1002,98 @@ void StatementTest::TestSqlIngestInts() {
                                           &reader.rows_affected, &error),
                 IsOkStatus(&error));
     ASSERT_THAT(reader.rows_affected,
-                ::testing::AnyOf(::testing::Eq(3), ::testing::Eq(-1)));
+                ::testing::AnyOf(::testing::Eq(values.size()), ::testing::Eq(-1)));
 
     ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
-    ASSERT_NO_FATAL_FAILURE(CompareSchema(&reader.schema.value,
-                                          {{"int64s", NANOARROW_TYPE_INT64, NULLABLE}}));
+    ArrowType round_trip_type = quirks()->IngestSelectRoundTripType(type);
+    ASSERT_NO_FATAL_FAILURE(
+        CompareSchema(&reader.schema.value, {{"col", round_trip_type, NULLABLE}}));
 
     ASSERT_NO_FATAL_FAILURE(reader.Next());
     ASSERT_NE(nullptr, reader.array->release);
-    ASSERT_EQ(3, reader.array->length);
+    ASSERT_EQ(values.size(), reader.array->length);
     ASSERT_EQ(1, reader.array->n_children);
 
-    ASSERT_NO_FATAL_FAILURE(
-        CompareArray<int64_t>(reader.array_view->children[0], {42, -42, std::nullopt}));
+    if (round_trip_type == type) {
+      // XXX: for now we can't compare values; we would need casting
+      ASSERT_NO_FATAL_FAILURE(
+          CompareArray<CType>(reader.array_view->children[0], values));
+    }
 
     ASSERT_NO_FATAL_FAILURE(reader.Next());
     ASSERT_EQ(nullptr, reader.array->release);
   }
 }
 
+template <typename CType>
+void StatementTest::TestSqlIngestNumericType(ArrowType type) {
+  std::vector<std::optional<CType>> values = {
+      std::nullopt,
+  };
+
+  if constexpr (std::is_floating_point_v<CType>) {
+    // XXX: sqlite and others seem to have trouble with extreme
+    // values. Likely a bug on our side, but for now, avoid them.
+    values.push_back(static_cast<CType>(-1.5));
+    values.push_back(static_cast<CType>(1.5));
+  } else {
+    values.push_back(std::numeric_limits<CType>::lowest());
+    values.push_back(std::numeric_limits<CType>::max());
+  }
+
+  return TestSqlIngestType(type, values);
+}
+
+void StatementTest::TestSqlIngestUInt8() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<uint8_t>(NANOARROW_TYPE_UINT8));
+}
+
+void StatementTest::TestSqlIngestUInt16() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<uint16_t>(NANOARROW_TYPE_UINT16));
+}
+
+void StatementTest::TestSqlIngestUInt32() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<uint32_t>(NANOARROW_TYPE_UINT32));
+}
+
+void StatementTest::TestSqlIngestUInt64() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<uint64_t>(NANOARROW_TYPE_UINT64));
+}
+
+void StatementTest::TestSqlIngestInt8() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<int8_t>(NANOARROW_TYPE_INT8));
+}
+
+void StatementTest::TestSqlIngestInt16() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<int16_t>(NANOARROW_TYPE_INT16));
+}
+
+void StatementTest::TestSqlIngestInt32() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<int32_t>(NANOARROW_TYPE_INT32));
+}
+
+void StatementTest::TestSqlIngestInt64() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<int64_t>(NANOARROW_TYPE_INT64));
+}
+
+void StatementTest::TestSqlIngestFloat32() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<float>(NANOARROW_TYPE_FLOAT));
+}
+
+void StatementTest::TestSqlIngestFloat64() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestNumericType<double>(NANOARROW_TYPE_DOUBLE));
+}
+
+void StatementTest::TestSqlIngestString() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestType<std::string>(
+      NANOARROW_TYPE_STRING, {std::nullopt, "", "", "1234", "例"}));
+}
+
+void StatementTest::TestSqlIngestBinary() {
+  ASSERT_NO_FATAL_FAILURE(TestSqlIngestType<std::string>(
+      NANOARROW_TYPE_BINARY, {std::nullopt, "", "\x00\x01\x02\x04", "\xFE\xFF"}));
+}
+
 void StatementTest::TestSqlIngestAppend() {
   if (!quirks()->supports_bulk_ingest()) {
     GTEST_SKIP();
@@ -1094,8 +1288,11 @@ void StatementTest::TestSqlIngestMultipleConnections() {
     ASSERT_THAT(AdbcConnectionInit(&connection2, &database, &error), IsOkStatus(&error));
     ASSERT_THAT(AdbcStatementNew(&connection2, &statement, &error), IsOkStatus(&error));
 
-    ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT * FROM bulk_ingest", &error),
-                IsOkStatus(&error));
+    ASSERT_THAT(
+        AdbcStatementSetSqlQuery(
+            &statement, "SELECT * FROM bulk_ingest ORDER BY \"int64s\" DESC NULLS LAST",
+            &error),
+        IsOkStatus(&error));
 
     {
       StreamReader reader;
@@ -1126,6 +1323,45 @@ void StatementTest::TestSqlIngestMultipleConnections() {
   }
 }
 
+void StatementTest::TestSqlIngestSample() {
+  if (!quirks()->supports_bulk_ingest()) {
+    GTEST_SKIP();
+  }
+
+  ASSERT_THAT(quirks()->EnsureSampleTable(&connection, "bulk_ingest", &error),
+              IsOkStatus(&error));
+
+  ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
+  ASSERT_THAT(AdbcStatementSetSqlQuery(
+                  &statement, "SELECT * FROM bulk_ingest ORDER BY int64s ASC NULLS FIRST",
+                  &error),
+              IsOkStatus(&error));
+  StreamReader reader;
+  ASSERT_THAT(AdbcStatementExecuteQuery(&statement, &reader.stream.value,
+                                        &reader.rows_affected, &error),
+              IsOkStatus(&error));
+  ASSERT_THAT(reader.rows_affected,
+              ::testing::AnyOf(::testing::Eq(3), ::testing::Eq(-1)));
+
+  ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
+  ASSERT_NO_FATAL_FAILURE(CompareSchema(&reader.schema.value,
+                                        {{"int64s", NANOARROW_TYPE_INT64, NULLABLE},
+                                         {"strings", NANOARROW_TYPE_STRING, NULLABLE}}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_NE(nullptr, reader.array->release);
+  ASSERT_EQ(3, reader.array->length);
+  ASSERT_EQ(2, reader.array->n_children);
+
+  ASSERT_NO_FATAL_FAILURE(
+      CompareArray<int64_t>(reader.array_view->children[0], {std::nullopt, -42, 42}));
+  ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(reader.array_view->children[1],
+                                                    {"", std::nullopt, "foo"}));
+
+  ASSERT_NO_FATAL_FAILURE(reader.Next());
+  ASSERT_EQ(nullptr, reader.array->release);
+}
+
 void StatementTest::TestSqlPartitionedInts() {
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
   ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT 42", &error),
@@ -1171,7 +1407,7 @@ void StatementTest::TestSqlPartitionedInts() {
   ASSERT_EQ(1, reader.array->length);
   ASSERT_EQ(1, reader.array->n_children);
 
-  switch (reader.fields[0].data_type) {
+  switch (reader.fields[0].type) {
     case NANOARROW_TYPE_INT32:
       ASSERT_NO_FATAL_FAILURE(
           CompareArray<int32_t>(reader.array_view->children[0], {42}));
@@ -1181,7 +1417,7 @@ void StatementTest::TestSqlPartitionedInts() {
           CompareArray<int64_t>(reader.array_view->children[0], {42}));
       break;
     default:
-      FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+      FAIL() << "Unexpected data type: " << reader.fields[0].type;
   }
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -1224,8 +1460,13 @@ void StatementTest::TestSqlPrepareSelectNoParams() {
   ASSERT_THAT(AdbcStatementExecuteQuery(&statement, &reader.stream.value,
                                         &reader.rows_affected, &error),
               IsOkStatus(&error));
-  ASSERT_THAT(reader.rows_affected,
-              ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+  if (quirks()->supports_rows_affected()) {
+    ASSERT_THAT(reader.rows_affected,
+                ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+  } else {
+    ASSERT_THAT(reader.rows_affected,
+                ::testing::Not(::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1))));
+  }
 
   ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
   ASSERT_EQ(1, reader.schema->n_children);
@@ -1235,7 +1476,7 @@ void StatementTest::TestSqlPrepareSelectNoParams() {
   ASSERT_EQ(1, reader.array->length);
   ASSERT_EQ(1, reader.array->n_children);
 
-  switch (reader.fields[0].data_type) {
+  switch (reader.fields[0].type) {
     case NANOARROW_TYPE_INT32:
       ASSERT_NO_FATAL_FAILURE(CompareArray<int32_t>(reader.array_view->children[0], {1}));
       break;
@@ -1243,7 +1484,7 @@ void StatementTest::TestSqlPrepareSelectNoParams() {
       ASSERT_NO_FATAL_FAILURE(CompareArray<int64_t>(reader.array_view->children[0], {1}));
       break;
     default:
-      FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+      FAIL() << "Unexpected data type: " << reader.fields[0].type;
   }
 
   ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -1300,7 +1541,10 @@ void StatementTest::TestSqlPrepareSelectParams() {
     auto start = nrows;
     auto end = nrows + reader.array->length;
 
-    switch (reader.fields[0].data_type) {
+    ASSERT_LT(start, expected_int32.size());
+    ASSERT_LE(end, expected_int32.size());
+
+    switch (reader.fields[0].type) {
       case NANOARROW_TYPE_INT32:
         ASSERT_NO_FATAL_FAILURE(CompareArray<int32_t>(
             reader.array_view->children[0],
@@ -1312,7 +1556,7 @@ void StatementTest::TestSqlPrepareSelectParams() {
             {expected_int64.begin() + start, expected_int64.begin() + end}));
         break;
       default:
-        FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+        FAIL() << "Unexpected data type: " << reader.fields[0].type;
     }
     ASSERT_NO_FATAL_FAILURE(CompareArray<std::string>(
         reader.array_view->children[1],
@@ -1326,7 +1570,8 @@ void StatementTest::TestSqlPrepareSelectParams() {
 }
 
 void StatementTest::TestSqlPrepareUpdate() {
-  if (!quirks()->supports_bulk_ingest()) {
+  if (!quirks()->supports_bulk_ingest() ||
+      !quirks()->supports_dynamic_parameter_binding()) {
     GTEST_SKIP();
   }
 
@@ -1404,7 +1649,8 @@ void StatementTest::TestSqlPrepareUpdateNoParams() {
 }
 
 void StatementTest::TestSqlPrepareUpdateStream() {
-  if (!quirks()->supports_bulk_ingest()) {
+  if (!quirks()->supports_bulk_ingest() ||
+      !quirks()->supports_dynamic_parameter_binding()) {
     GTEST_SKIP();
   }
 
@@ -1548,8 +1794,13 @@ void StatementTest::TestSqlQueryInts() {
     ASSERT_THAT(AdbcStatementExecuteQuery(&statement, &reader.stream.value,
                                           &reader.rows_affected, &error),
                 IsOkStatus(&error));
-    ASSERT_THAT(reader.rows_affected,
-                ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    if (quirks()->supports_rows_affected()) {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    } else {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::Not(::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1))));
+    }
 
     ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
     ASSERT_EQ(1, reader.schema->n_children);
@@ -1559,7 +1810,7 @@ void StatementTest::TestSqlQueryInts() {
     ASSERT_EQ(1, reader.array->length);
     ASSERT_EQ(1, reader.array->n_children);
 
-    switch (reader.fields[0].data_type) {
+    switch (reader.fields[0].type) {
       case NANOARROW_TYPE_INT32:
         ASSERT_NO_FATAL_FAILURE(
             CompareArray<int32_t>(reader.array_view->children[0], {42}));
@@ -1569,7 +1820,7 @@ void StatementTest::TestSqlQueryInts() {
             CompareArray<int64_t>(reader.array_view->children[0], {42}));
         break;
       default:
-        FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+        FAIL() << "Unexpected data type: " << reader.fields[0].type;
     }
 
     ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -1581,7 +1832,7 @@ void StatementTest::TestSqlQueryInts() {
 
 void StatementTest::TestSqlQueryFloats() {
   ASSERT_THAT(AdbcStatementNew(&connection, &statement, &error), IsOkStatus(&error));
-  ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT CAST(1.0 AS FLOAT)", &error),
+  ASSERT_THAT(AdbcStatementSetSqlQuery(&statement, "SELECT CAST(1.5 AS FLOAT)", &error),
               IsOkStatus(&error));
 
   {
@@ -1589,8 +1840,13 @@ void StatementTest::TestSqlQueryFloats() {
     ASSERT_THAT(AdbcStatementExecuteQuery(&statement, &reader.stream.value,
                                           &reader.rows_affected, &error),
                 IsOkStatus(&error));
-    ASSERT_THAT(reader.rows_affected,
-                ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    if (quirks()->supports_rows_affected()) {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    } else {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::Not(::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1))));
+    }
 
     ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
     ASSERT_EQ(1, reader.schema->n_children);
@@ -1602,17 +1858,17 @@ void StatementTest::TestSqlQueryFloats() {
 
     ASSERT_FALSE(ArrowArrayViewIsNull(&reader.array_view.value, 0));
     ASSERT_FALSE(ArrowArrayViewIsNull(reader.array_view->children[0], 0));
-    switch (reader.fields[0].data_type) {
+    switch (reader.fields[0].type) {
       case NANOARROW_TYPE_FLOAT:
         ASSERT_NO_FATAL_FAILURE(
-            CompareArray<float>(reader.array_view->children[0], {1.0f}));
+            CompareArray<float>(reader.array_view->children[0], {1.5f}));
         break;
       case NANOARROW_TYPE_DOUBLE:
         ASSERT_NO_FATAL_FAILURE(
-            CompareArray<double>(reader.array_view->children[0], {1.0}));
+            CompareArray<double>(reader.array_view->children[0], {1.5}));
         break;
       default:
-        FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+        FAIL() << "Unexpected data type: " << reader.fields[0].type;
     }
 
     ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -1632,8 +1888,13 @@ void StatementTest::TestSqlQueryStrings() {
     ASSERT_THAT(AdbcStatementExecuteQuery(&statement, &reader.stream.value,
                                           &reader.rows_affected, &error),
                 IsOkStatus(&error));
-    ASSERT_THAT(reader.rows_affected,
-                ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    if (quirks()->supports_rows_affected()) {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1)));
+    } else {
+      ASSERT_THAT(reader.rows_affected,
+                  ::testing::Not(::testing::AnyOf(::testing::Eq(1), ::testing::Eq(-1))));
+    }
 
     ASSERT_NO_FATAL_FAILURE(reader.GetSchema());
     ASSERT_EQ(1, reader.schema->n_children);
@@ -1645,14 +1906,15 @@ void StatementTest::TestSqlQueryStrings() {
 
     ASSERT_FALSE(ArrowArrayViewIsNull(&reader.array_view.value, 0));
     ASSERT_FALSE(ArrowArrayViewIsNull(reader.array_view->children[0], 0));
-    switch (reader.fields[0].data_type) {
+    switch (reader.fields[0].type) {
+      case NANOARROW_TYPE_LARGE_STRING:
       case NANOARROW_TYPE_STRING: {
         ASSERT_NO_FATAL_FAILURE(
             CompareArray<std::string>(reader.array_view->children[0], {"SaShiSuSeSo"}));
         break;
       }
       default:
-        FAIL() << "Unexpected data type: " << reader.fields[0].data_type;
+        FAIL() << "Unexpected data type: " << reader.fields[0].type;
     }
 
     ASSERT_NO_FATAL_FAILURE(reader.Next());
@@ -1674,7 +1936,7 @@ void StatementTest::TestSqlQueryErrors() {
 }
 
 void StatementTest::TestTransactions() {
-  if (!quirks()->supports_transactions()) {
+  if (!quirks()->supports_transactions() || quirks()->ddl_implicit_commit_txn()) {
     GTEST_SKIP();
   }
 
@@ -1725,10 +1987,7 @@ void StatementTest::TestTransactions() {
               })(),
               ::testing::Not(IsOkStatus(&error)));
 
-  if (error.release) {
-    std::cerr << "Failure message: " << error.message << std::endl;
-    error.release(&error);
-  }
+  if (error.release) error.release(&error);
 
   // Rollback
   ASSERT_THAT(AdbcConnectionRollback(&connection, &error), IsOkStatus(&error));
