@@ -18,10 +18,13 @@
 #ifndef ADBC_VALIDATION_H
 #define ADBC_VALIDATION_H
 
+#include <optional>
 #include <string>
+#include <vector>
 
 #include <adbc.h>
 #include <gtest/gtest.h>
+#include <nanoarrow/nanoarrow.h>
 
 namespace adbc_validation {
 
@@ -44,6 +47,13 @@ class DriverQuirks {
     return ADBC_STATUS_OK;
   }
 
+  /// \brief Drop the given view. Used by tests to reset state.
+  virtual AdbcStatusCode DropView(struct AdbcConnection* connection,
+                                  const std::string& name,
+                                  struct AdbcError* error) const {
+    return ADBC_STATUS_NOT_IMPLEMENTED;
+  }
+
   virtual AdbcStatusCode EnsureSampleTable(struct AdbcConnection* connection,
                                            const std::string& name,
                                            struct AdbcError* error) const;
@@ -57,9 +67,24 @@ class DriverQuirks {
                                            const std::string& name,
                                            struct AdbcError* error) const;
 
+  /// \brief Get the statement to create a table with a primary key, or nullopt if not
+  /// supported.
+  ///
+  /// The table should have one column:
+  /// - "id" with Arrow type int64 (primary key)
+  virtual std::optional<std::string> PrimaryKeyTableDdl(std::string_view name) const {
+    return std::nullopt;
+  }
+
   /// \brief Return the SQL to reference the bind parameter of the given index
   virtual std::string BindParameter(int index) const { return "?"; }
 
+  /// \brief For a given Arrow type of ingested data, what Arrow type
+  ///   will the database return when that column is selected?
+  virtual ArrowType IngestSelectRoundTripType(ArrowType ingest_type) const {
+    return ingest_type;
+  }
+
   /// \brief Whether two statements can be used at the same time on a
   ///   single connection
   virtual bool supports_concurrent_statements() const { return false; }
@@ -70,6 +95,9 @@ class DriverQuirks {
   /// \brief Whether transaction methods are implemented
   virtual bool supports_transactions() const { return true; }
 
+  /// \brief Whether or not DDL implicitly commits a transaction
+  virtual bool ddl_implicit_commit_txn() const { return false; }
+
   /// \brief Whether GetSqlInfo is implemented
   virtual bool supports_get_sql_info() const { return true; }
 
@@ -81,6 +109,15 @@ class DriverQuirks {
 
   /// \brief Whether dynamic parameter bindings are supported for prepare
   virtual bool supports_dynamic_parameter_binding() const { return true; }
+
+  /// \brief Whether ExecuteQuery sets rows_affected appropriately
+  virtual bool supports_rows_affected() const { return true; }
+
+  /// \brief Default catalog to use for tests
+  virtual std::string catalog() const { return ""; }
+
+  /// \brief Default Schema to use for tests
+  virtual std::string db_schema() const { return ""; }
 };
 
 class DatabaseTest {
@@ -130,6 +167,7 @@ class ConnectionTest {
   void TestMetadataGetObjectsTablesTypes();
   void TestMetadataGetObjectsColumns();
   void TestMetadataGetObjectsConstraints();
+  void TestMetadataGetObjectsPrimaryKey();
 
  protected:
   struct AdbcError error;
@@ -155,7 +193,10 @@ class ConnectionTest {
     TestMetadataGetObjectsTablesTypes();                                              \
   }                                                                                   \
   TEST_F(FIXTURE, MetadataGetObjectsColumns) { TestMetadataGetObjectsColumns(); }     \
-  TEST_F(FIXTURE, MetadataGetObjectsConstraints) { TestMetadataGetObjectsConstraints(); }
+  TEST_F(FIXTURE, MetadataGetObjectsConstraints) {                                    \
+    TestMetadataGetObjectsConstraints();                                              \
+  }                                                                                   \
+  TEST_F(FIXTURE, MetadataGetObjectsPrimaryKey) { TestMetadataGetObjectsPrimaryKey(); }
 
 class StatementTest {
  public:
@@ -168,11 +209,32 @@ class StatementTest {
   void TestNewInit();
   void TestRelease();
 
-  // TODO: these should be parameterized tests
-  void TestSqlIngestInts();
+  // ---- Type-specific tests --------------------
+
+  // Integers
+  void TestSqlIngestInt8();
+  void TestSqlIngestInt16();
+  void TestSqlIngestInt32();
+  void TestSqlIngestInt64();
+  void TestSqlIngestUInt8();
+  void TestSqlIngestUInt16();
+  void TestSqlIngestUInt32();
+  void TestSqlIngestUInt64();
+
+  // Floats
+  void TestSqlIngestFloat32();
+  void TestSqlIngestFloat64();
+
+  // Strings
+  void TestSqlIngestString();
+  void TestSqlIngestBinary();
+
+  // ---- End Type-specific tests ----------------
+
   void TestSqlIngestAppend();
   void TestSqlIngestErrors();
   void TestSqlIngestMultipleConnections();
+  void TestSqlIngestSample();
 
   void TestSqlPartitionedInts();
 
@@ -201,6 +263,12 @@ class StatementTest {
   struct AdbcDatabase database;
   struct AdbcConnection connection;
   struct AdbcStatement statement;
+
+  template <typename CType>
+  void TestSqlIngestType(ArrowType type, const std::vector<std::optional<CType>>& values);
+
+  template <typename CType>
+  void TestSqlIngestNumericType(ArrowType type);
 };
 
 #define ADBCV_TEST_STATEMENT(FIXTURE)                                                   \
@@ -208,10 +276,22 @@ class StatementTest {
                 ADBCV_STRINGIFY(FIXTURE) " must inherit from StatementTest");           \
   TEST_F(FIXTURE, NewInit) { TestNewInit(); }                                           \
   TEST_F(FIXTURE, Release) { TestRelease(); }                                           \
-  TEST_F(FIXTURE, SqlIngestInts) { TestSqlIngestInts(); }                               \
+  TEST_F(FIXTURE, SqlIngestInt8) { TestSqlIngestInt8(); }                               \
+  TEST_F(FIXTURE, SqlIngestInt16) { TestSqlIngestInt16(); }                             \
+  TEST_F(FIXTURE, SqlIngestInt32) { TestSqlIngestInt32(); }                             \
+  TEST_F(FIXTURE, SqlIngestInt64) { TestSqlIngestInt64(); }                             \
+  TEST_F(FIXTURE, SqlIngestUInt8) { TestSqlIngestUInt8(); }                             \
+  TEST_F(FIXTURE, SqlIngestUInt16) { TestSqlIngestUInt16(); }                           \
+  TEST_F(FIXTURE, SqlIngestUInt32) { TestSqlIngestUInt32(); }                           \
+  TEST_F(FIXTURE, SqlIngestUInt64) { TestSqlIngestUInt64(); }                           \
+  TEST_F(FIXTURE, SqlIngestFloat32) { TestSqlIngestFloat32(); }                         \
+  TEST_F(FIXTURE, SqlIngestFloat64) { TestSqlIngestFloat64(); }                         \
+  TEST_F(FIXTURE, SqlIngestString) { TestSqlIngestString(); }                           \
+  TEST_F(FIXTURE, SqlIngestBinary) { TestSqlIngestBinary(); }                           \
   TEST_F(FIXTURE, SqlIngestAppend) { TestSqlIngestAppend(); }                           \
   TEST_F(FIXTURE, SqlIngestErrors) { TestSqlIngestErrors(); }                           \
   TEST_F(FIXTURE, SqlIngestMultipleConnections) { TestSqlIngestMultipleConnections(); } \
+  TEST_F(FIXTURE, SqlIngestSample) { TestSqlIngestSample(); }                           \
   TEST_F(FIXTURE, SqlPartitionedInts) { TestSqlPartitionedInts(); }                     \
   TEST_F(FIXTURE, SqlPrepareGetParameterSchema) { TestSqlPrepareGetParameterSchema(); } \
   TEST_F(FIXTURE, SqlPrepareSelectNoParams) { TestSqlPrepareSelectNoParams(); }         \
