@@ -23,6 +23,7 @@ set -e
 : ${BUILD_DRIVER_MANAGER:=${BUILD_ALL}}
 : ${BUILD_DRIVER_POSTGRESQL:=${BUILD_ALL}}
 : ${BUILD_DRIVER_SQLITE:=${BUILD_ALL}}
+: ${BUILD_DRIVER_SNOWFLAKE:=${BUILD_ALL}}
 
 test_subproject() {
     local -r source_dir=${1}
@@ -63,6 +64,10 @@ main() {
     if [[ "${BUILD_DRIVER_SQLITE}" -gt 0 ]]; then
         test_subproject "${source_dir}" "${install_dir}" adbc_driver_sqlite
     fi
+
+    if [[ "${BUILD_DRIVER_SNOWFLAKE}" -gt 0 ]]; then
+        test_subproject "${source_dir}" "${install_dir}" adbc_driver_snowflake
+    fi
 }
 
 main "$@"
