@@ -27,11 +27,13 @@ update_versions() {
       local version=${base_version}
       local conda_version=${base_version}
       local docs_version=${base_version}
+      local r_version=${base_version}
       ;;
     snapshot)
       local version=${next_version}-SNAPSHOT
       local conda_version=${next_version}
       local docs_version="${next_version} (dev)"
+      local r_version="${base_version}.9000"
       ;;
   esac
   local major_version=${version%%.*}
@@ -42,12 +44,20 @@ update_versions() {
   git add cmake_modules/AdbcVersion.cmake
   popd
 
+  sed -i.bak -E "s/set ADBC_VERSION_DLL_SUFFIX=.+/set ADBC_VERSION_DLL_SUFFIX=.${version}/" "${ADBC_DIR}/ci/scripts/python_wheel_windows_build.bat"
+  rm "${ADBC_DIR}/ci/scripts/python_wheel_windows_build.bat.bak"
+  git add "${ADBC_DIR}/ci/scripts/python_wheel_windows_build.bat"
+
   pushd "${ADBC_DIR}/ci/conda/"
   sed -i.bak -E "s/version: .+/version: ${conda_version}/g" meta.yaml
   rm meta.yaml.bak
   git add meta.yaml
   popd
 
+  sed -i.bak -E "s/version = \".+\"/version = \"${version}\"/" "${ADBC_DIR}/csharp/Directory.Build.props"
+  rm "${ADBC_DIR}/csharp/Directory.Build.props.bak"
+  git add "${ADBC_DIR}/csharp/Directory.Build.props"
+
   sed -i.bak -E "s/release = \".+\"/release = \"${docs_version}\"/g" "${ADBC_DIR}/docs/source/conf.py"
   rm "${ADBC_DIR}/docs/source/conf.py.bak"
   git add "${ADBC_DIR}/docs/source/conf.py"
@@ -68,6 +78,16 @@ update_versions() {
   rm "${ADBC_DIR}/ruby/lib/adbc/version.rb.bak"
   git add "${ADBC_DIR}/ruby/lib/adbc/version.rb"
 
+  for desc_file in $(find "${ADBC_DIR}/r" -name DESCRIPTION); do
+    sed -i.bak -E "s/Version:.*$/Version: ${r_version}/" "${desc_file}"
+    rm "${desc_file}.bak"
+    git add "${desc_file}"
+  done
+
+  sed -i.bak -E "s/^version = \".+\"/version = \"${version}\"/" "${ADBC_DIR}/rust/Cargo.toml"
+  rm "${ADBC_DIR}/rust/Cargo.toml.bak"
+  git add "${ADBC_DIR}/rust/Cargo.toml"
+
   if [ ${type} = "release" ]; then
     pushd "${ADBC_DIR}/ci/linux-packages"
     rake version:update VERSION=${version}
