@@ -45,9 +45,21 @@ echo "::group::Prepare repository"
 
 export DEBIAN_FRONTEND=noninteractive
 
-APT_INSTALL="apt install -y -V --no-install-recommends"
+retry() {
+  local n_tries=2
+  while [ ${n_tries} -gt 0 ]; do
+    if "$@"; then
+      return
+    fi
+    n_tries=$((${n_tries} - 1))
+  done
+  "$@"
+}
 
-apt update
+APT_UPDATE="retry apt update --error-on=any"
+APT_INSTALL="retry apt install -y -V --no-install-recommends"
+
+${APT_UPDATE}
 ${APT_INSTALL} \
   ca-certificates \
   curl \
@@ -127,7 +139,7 @@ else
   esac
 fi
 
-apt update
+${APT_UPDATE}
 
 echo "::endgroup::"
 
@@ -166,6 +178,14 @@ ${APT_INSTALL} libadbc-driver-sqlite-dev=${package_version}
 echo "::endgroup::"
 
 
+echo "::group::Test ADBC Flight SQL Driver"
+${APT_INSTALL} libadbc-driver-flightsql-dev=${package_version}
+echo "::endgroup::"
+
+echo "::group::Test ADBC Snowflake Driver"
+${APT_INSTALL} libadbc-driver-snowflake-dev=${package_version}
+echo "::endgroup::"
+
 echo "::group::Test ADBC GLib"
 export G_DEBUG=fatal-warnings
 
