@@ -15,13 +15,18 @@
 # specific language governing permissions and limitations
 # under the License.
 
+import sys
+from pathlib import Path
+
+sys.path.append(str(Path("./ext").resolve()))
+
 # -- Project information -----------------------------------------------------
 # https://www.sphinx-doc.org/en/master/usage/configuration.html#project-information
 
 project = "ADBC"
 copyright = "2022, Apache Arrow Developers"
 author = "the Apache Arrow Developers"
-release = "0.2.0"
+release = "0.5.1"
 # Needed to generate version switcher
 version = release
 
@@ -30,6 +35,7 @@ version = release
 
 exclude_patterns = []
 extensions = [
+    "adbc_cookbook",
     "breathe",
     "numpydoc",
     "sphinx.ext.autodoc",
@@ -37,6 +43,7 @@ extensions = [
     "sphinx.ext.intersphinx",
     "sphinx_copybutton",
     "sphinx_design",
+    "sphinxcontrib.mermaid",
 ]
 templates_path = ["_templates"]
 
@@ -101,6 +108,10 @@ intersphinx_mapping = {
     "arrow": ("https://arrow.apache.org/docs/", None),
 }
 
+# -- Options for mermaid -----------------------------------------------------
+
+mermaid_d3_zoom = True
+
 # -- Options for numpydoc ----------------------------------------------------
 
 numpydoc_class_members_toctree = False
