@@ -21,12 +21,12 @@
 // It can be used to register a driver for database/sql by importing
 // github.com/apache/arrow-adbc/go/adbc/sqldriver and running:
 //
-//     sql.Register("flightsql", sqldriver.Driver{flightsql.Driver{}})
+//	sql.Register("flightsql", sqldriver.Driver{flightsql.Driver{}})
 //
 // You can then open a flightsql connection with the database/sql
 // standard package by using:
 //
-//     db, err := sql.Open("flightsql", "uri=<flight sql db url>")
+//	db, err := sql.Open("flightsql", "uri=<flight sql db url>")
 //
 // The URI passed *must* contain a scheme, most likely "grpc+tcp://"
 package flightsql
@@ -41,26 +41,29 @@ import (
 	"io"
 	"math"
 	"net/url"
-	"regexp"
 	"runtime/debug"
 	"strconv"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/apache/arrow-adbc/go/adbc"
-	"github.com/apache/arrow/go/v12/arrow"
-	"github.com/apache/arrow/go/v12/arrow/array"
-	"github.com/apache/arrow/go/v12/arrow/flight"
-	"github.com/apache/arrow/go/v12/arrow/flight/flightsql"
-	"github.com/apache/arrow/go/v12/arrow/flight/flightsql/schema_ref"
-	"github.com/apache/arrow/go/v12/arrow/ipc"
-	"github.com/apache/arrow/go/v12/arrow/memory"
+	"github.com/apache/arrow-adbc/go/adbc/driver/internal"
+	"github.com/apache/arrow/go/v13/arrow"
+	"github.com/apache/arrow/go/v13/arrow/array"
+	"github.com/apache/arrow/go/v13/arrow/flight"
+	"github.com/apache/arrow/go/v13/arrow/flight/flightsql"
+	"github.com/apache/arrow/go/v13/arrow/flight/flightsql/schema_ref"
+	"github.com/apache/arrow/go/v13/arrow/ipc"
+	"github.com/apache/arrow/go/v13/arrow/memory"
 	"github.com/bluele/gcache"
 	"golang.org/x/exp/maps"
 	"google.golang.org/grpc"
+	grpccodes "google.golang.org/grpc/codes"
 	"google.golang.org/grpc/credentials"
 	"google.golang.org/grpc/credentials/insecure"
 	"google.golang.org/grpc/metadata"
+	grpcstatus "google.golang.org/grpc/status"
 	"google.golang.org/protobuf/proto"
 )
 
@@ -77,14 +80,21 @@ const (
 	OptionTimeoutQuery        = "adbc.flight.sql.rpc.timeout_seconds.query"
 	OptionTimeoutUpdate       = "adbc.flight.sql.rpc.timeout_seconds.update"
 	OptionRPCCallHeaderPrefix = "adbc.flight.sql.rpc.call_header."
+	OptionCookieMiddleware    = "adbc.flight.sql.rpc.with_cookie_middleware"
 	infoDriverName            = "ADBC Flight SQL Driver - Go"
 )
 
 var (
 	infoDriverVersion      string
 	infoDriverArrowVersion string
+	infoSupportedCodes     []adbc.InfoCode
 )
 
+var errNoTransactionSupport = adbc.Error{
+	Msg:  "[Flight SQL] server does not report transaction support",
+	Code: adbc.StatusNotImplemented,
+}
+
 func init() {
 	if info, ok := debug.ReadBuildInfo(); ok {
 		for _, dep := range info.Deps {
@@ -96,6 +106,23 @@ func init() {
 			}
 		}
 	}
+	// XXX: Deps not populated in tests
+	// https://github.com/golang/go/issues/33976
+	if infoDriverVersion == "" {
+		infoDriverVersion = "(unknown or development build)"
+	}
+	if infoDriverArrowVersion == "" {
+		infoDriverArrowVersion = "(unknown or development build)"
+	}
+
+	infoSupportedCodes = []adbc.InfoCode{
+		adbc.InfoDriverName,
+		adbc.InfoDriverVersion,
+		adbc.InfoDriverArrowVersion,
+		adbc.InfoVendorName,
+		adbc.InfoVendorVersion,
+		adbc.InfoVendorArrowVersion,
+	}
 }
 
 func getTimeoutOptionValue(v string) (time.Duration, error) {
@@ -150,6 +177,7 @@ func (d *dbDialOpts) rebuild() {
 	d.opts = []grpc.DialOption{
 		grpc.WithDefaultCallOptions(grpc.MaxCallRecvMsgSize(d.maxMsgSize),
 			grpc.MaxCallSendMsgSize(d.maxMsgSize)),
+		grpc.WithUserAgent("ADBC Flight SQL Driver " + infoDriverVersion),
 	}
 	if d.block {
 		d.opts = append(d.opts, grpc.WithBlock())
@@ -157,12 +185,13 @@ func (d *dbDialOpts) rebuild() {
 }
 
 type database struct {
-	uri        *url.URL
-	creds      credentials.TransportCredentials
-	user, pass string
-	hdrs       metadata.MD
-	timeout    timeoutOption
-	dialOpts   dbDialOpts
+	uri           *url.URL
+	creds         credentials.TransportCredentials
+	user, pass    string
+	hdrs          metadata.MD
+	timeout       timeoutOption
+	dialOpts      dbDialOpts
+	enableCookies bool
 
 	alloc memory.Allocator
 }
@@ -300,6 +329,7 @@ func (d *database) SetOptions(cnOptions map[string]string) error {
 		}
 		delete(cnOptions, OptionWithBlock)
 	}
+
 	if val, ok := cnOptions[OptionWithMaxMsgSize]; ok {
 		var err error
 		var size int
@@ -319,6 +349,20 @@ func (d *database) SetOptions(cnOptions map[string]string) error {
 	}
 	d.dialOpts.rebuild()
 
+	if val, ok := cnOptions[OptionCookieMiddleware]; ok {
+		if val == adbc.OptionValueEnabled {
+			d.enableCookies = true
+		} else if val == adbc.OptionValueDisabled {
+			d.enableCookies = false
+		} else {
+			return adbc.Error{
+				Msg:  fmt.Sprintf("Invalid value for database option '%s': '%s'", OptionCookieMiddleware, val),
+				Code: adbc.StatusInvalidArgument,
+			}
+		}
+		delete(cnOptions, OptionCookieMiddleware)
+	}
+
 	for key, val := range cnOptions {
 		if strings.HasPrefix(key, OptionRPCCallHeaderPrefix) {
 			d.hdrs.Append(strings.TrimPrefix(key, OptionRPCCallHeaderPrefix), val)
@@ -374,25 +418,139 @@ func unaryTimeoutInterceptor(ctx context.Context, method string, req, reply any,
 	return invoker(ctx, method, req, reply, cc, opts...)
 }
 
+type streamEventType int
+
+const (
+	receiveEndEvent streamEventType = iota
+	errorEvent
+)
+
+type streamEvent struct {
+	Type streamEventType
+	Err  error
+}
+
+type wrappedClientStream struct {
+	grpc.ClientStream
+
+	desc       *grpc.StreamDesc
+	events     chan streamEvent
+	eventsDone chan struct{}
+}
+
+func (w *wrappedClientStream) RecvMsg(m any) error {
+	err := w.ClientStream.RecvMsg(m)
+
+	switch {
+	case err == nil && !w.desc.ServerStreams:
+		w.sendStreamEvent(receiveEndEvent, nil)
+	case err == io.EOF:
+		w.sendStreamEvent(receiveEndEvent, nil)
+	case err != nil:
+		w.sendStreamEvent(errorEvent, err)
+	}
+
+	return err
+}
+
+func (w *wrappedClientStream) SendMsg(m any) error {
+	err := w.ClientStream.SendMsg(m)
+	if err != nil {
+		w.sendStreamEvent(errorEvent, err)
+	}
+	return err
+}
+
+func (w *wrappedClientStream) Header() (metadata.MD, error) {
+	md, err := w.ClientStream.Header()
+	if err != nil {
+		w.sendStreamEvent(errorEvent, err)
+	}
+	return md, err
+}
+
+func (w *wrappedClientStream) CloseSend() error {
+	err := w.ClientStream.CloseSend()
+	if err != nil {
+		w.sendStreamEvent(errorEvent, err)
+	}
+	return err
+}
+
+func (w *wrappedClientStream) sendStreamEvent(eventType streamEventType, err error) {
+	select {
+	case <-w.eventsDone:
+	case w.events <- streamEvent{Type: eventType, Err: err}:
+	}
+}
+
 func streamTimeoutInterceptor(ctx context.Context, desc *grpc.StreamDesc, cc *grpc.ClientConn, method string, streamer grpc.Streamer, opts ...grpc.CallOption) (grpc.ClientStream, error) {
 	if tm, ok := getTimeout(method, opts); ok {
 		ctx, cancel := context.WithTimeout(ctx, tm)
-		defer cancel()
-		return streamer(ctx, desc, cc, method, opts...)
+		s, err := streamer(ctx, desc, cc, method, opts...)
+		if err != nil {
+			defer cancel()
+			return s, err
+		}
+
+		events, eventsDone := make(chan streamEvent), make(chan struct{})
+		go func() {
+			defer close(eventsDone)
+			defer cancel()
+
+			for {
+				select {
+				case event := <-events:
+					// split by event type in case we want to add more logging
+					// or even adding in some telemetry in the future.
+					// Errors will already be propagated by the RecvMsg, SendMsg
+					// methods.
+					switch event.Type {
+					case receiveEndEvent:
+						return
+					case errorEvent:
+						return
+					}
+				case <-ctx.Done():
+					return
+				}
+			}
+		}()
+
+		stream := &wrappedClientStream{
+			ClientStream: s,
+			desc:         desc,
+			events:       events,
+			eventsDone:   eventsDone,
+		}
+		return stream, nil
 	}
 
 	return streamer(ctx, desc, cc, method, opts...)
 }
 
 type bearerAuthMiddleware struct {
-	hdrs metadata.MD
+	mutex sync.RWMutex
+	hdrs  metadata.MD
 }
 
 func (b *bearerAuthMiddleware) StartCall(ctx context.Context) context.Context {
 	md, _ := metadata.FromOutgoingContext(ctx)
+	b.mutex.RLock()
+	defer b.mutex.RUnlock()
 	return metadata.NewOutgoingContext(ctx, metadata.Join(md, b.hdrs))
 }
 
+func (b *bearerAuthMiddleware) HeadersReceived(ctx context.Context, md metadata.MD) {
+	// apache/arrow-adbc#584
+	headers := md.Get("authorization")
+	if len(headers) > 0 {
+		b.mutex.Lock()
+		defer b.mutex.Unlock()
+		b.hdrs.Set("authorization", headers...)
+	}
+}
+
 func getFlightClient(ctx context.Context, loc string, d *database) (*flightsql.Client, error) {
 	authMiddle := &bearerAuthMiddleware{hdrs: d.hdrs.Copy()}
 	middleware := []flight.ClientMiddleware{
@@ -403,17 +561,26 @@ func getFlightClient(ctx context.Context, loc string, d *database) (*flightsql.C
 		},
 	}
 
+	if d.enableCookies {
+		middleware = append(middleware, flight.NewClientCookieMiddleware())
+	}
+
 	uri, err := url.Parse(loc)
 	if err != nil {
 		return nil, adbc.Error{Msg: fmt.Sprintf("Invalid URI '%s': %s", loc, err), Code: adbc.StatusInvalidArgument}
 	}
 	creds := d.creds
+
+	target := uri.Host
 	if uri.Scheme == "grpc" || uri.Scheme == "grpc+tcp" {
 		creds = insecure.NewCredentials()
+	} else if uri.Scheme == "grpc+unix" {
+		creds = insecure.NewCredentials()
+		target = "unix:" + uri.Path
 	}
 	dialOpts := append(d.dialOpts.opts, grpc.WithTransportCredentials(creds))
 
-	cl, err := flightsql.NewClient(uri.Host, nil, middleware, dialOpts...)
+	cl, err := flightsql.NewClient(target, nil, middleware, dialOpts...)
 	if err != nil {
 		return nil, adbc.Error{
 			Msg:  err.Error(),
@@ -432,6 +599,7 @@ func getFlightClient(ctx context.Context, loc string, d *database) (*flightsql.C
 		}
 
 		if md, ok := metadata.FromOutgoingContext(ctx); ok {
+			// No need to worry about lock here since we are sole owner
 			authMiddle.hdrs.Set("authorization", md.Get("Authorization")[0])
 		}
 	}
@@ -439,6 +607,10 @@ func getFlightClient(ctx context.Context, loc string, d *database) (*flightsql.C
 	return cl, nil
 }
 
+type support struct {
+	transactions bool
+}
+
 func (d *database) Open(ctx context.Context) (adbc.Connection, error) {
 	cl, err := getFlightClient(ctx, d.uri.String(), d)
 	if err != nil {
@@ -465,8 +637,52 @@ func (d *database) Open(ctx context.Context) (adbc.Connection, error) {
 			conn := client.(*flightsql.Client)
 			conn.Close()
 		}).Build()
+
+	var cnxnSupport support
+
+	info, err := cl.GetSqlInfo(ctx, []flightsql.SqlInfo{flightsql.SqlInfoFlightSqlServerTransaction}, d.timeout)
+	// ignore this if it fails
+	if err == nil {
+		const int32code = 3
+
+		for _, endpoint := range info.Endpoint {
+			rdr, err := doGet(ctx, cl, endpoint, cache, d.timeout)
+			if err != nil {
+				continue
+			}
+			defer rdr.Release()
+
+			for rdr.Next() {
+				rec := rdr.Record()
+				codes := rec.Column(0).(*array.Uint32)
+				values := rec.Column(1).(*array.DenseUnion)
+				int32Value := values.Field(int32code).(*array.Int32)
+
+				for i := 0; i < int(rec.NumRows()); i++ {
+					switch codes.Value(i) {
+					case uint32(flightsql.SqlInfoFlightSqlServerTransaction):
+						if values.TypeCode(i) != int32code {
+							continue
+						}
+
+						idx := values.ValueOffset(i)
+						if !int32Value.IsValid(int(idx)) {
+							continue
+						}
+
+						value := int32Value.Value(int(idx))
+						cnxnSupport.transactions =
+							value == int32(flightsql.SqlTransactionTransaction) ||
+								value == int32(flightsql.SqlTransactionSavepoint)
+					}
+				}
+			}
+		}
+	}
+
 	return &cnxn{cl: cl, db: d, clientCache: cache,
-		hdrs: make(metadata.MD), timeouts: d.timeout}, nil
+		hdrs: make(metadata.MD), timeouts: d.timeout,
+		supportInfo: cnxnSupport}, nil
 }
 
 type cnxn struct {
@@ -476,6 +692,8 @@ type cnxn struct {
 	clientCache gcache.Cache
 	hdrs        metadata.MD
 	timeouts    timeoutOption
+	txn         *flightsql.Txn
+	supportInfo support
 }
 
 var adbcToFlightSQLInfo = map[adbc.InfoCode]flightsql.SqlInfo{
@@ -554,10 +772,51 @@ func (c *cnxn) SetOption(key, value string) error {
 		}
 		c.timeouts.updateTimeout = timeout
 	case adbc.OptionKeyAutoCommit:
-		return adbc.Error{
-			Msg:  "[Flight SQL] transactions not yet supported",
-			Code: adbc.StatusNotImplemented,
+		autocommit := true
+		switch value {
+		case adbc.OptionValueEnabled:
+		case adbc.OptionValueDisabled:
+			autocommit = false
+		default:
+			return adbc.Error{
+				Msg:  "[Flight SQL] invalid value for option " + key + ": " + value,
+				Code: adbc.StatusInvalidArgument,
+			}
 		}
+
+		if autocommit && c.txn == nil {
+			// no-op don't even error if the server didn't support transactions
+			return nil
+		}
+
+		if !c.supportInfo.transactions {
+			return errNoTransactionSupport
+		}
+
+		ctx := metadata.NewOutgoingContext(context.Background(), c.hdrs)
+		var err error
+		if c.txn != nil {
+			if err = c.txn.Commit(ctx, c.timeouts); err != nil {
+				return adbc.Error{
+					Msg:  "[Flight SQL] failed to update autocommit: " + err.Error(),
+					Code: adbc.StatusIO,
+				}
+			}
+		}
+
+		if autocommit {
+			c.txn = nil
+			return nil
+		}
+
+		if c.txn, err = c.cl.BeginTransaction(ctx, c.timeouts); err != nil {
+			return adbc.Error{
+				Msg:  "[Flight SQL] failed to update autocommit: " + err.Error(),
+				Code: adbc.StatusIO,
+			}
+		}
+		return nil
+
 	default:
 		return adbc.Error{
 			Msg:  "[Flight SQL] unknown connection option",
@@ -572,21 +831,21 @@ func (c *cnxn) SetOption(key, value string) error {
 //
 // The result is an Arrow dataset with the following schema:
 //
-//    Field Name									| Field Type
-//    ----------------------------|-----------------------------
-//    info_name					   				| uint32 not null
-//    info_value									| INFO_SCHEMA
+//	Field Name									| Field Type
+//	----------------------------|-----------------------------
+//	info_name					   				| uint32 not null
+//	info_value									| INFO_SCHEMA
 //
 // INFO_SCHEMA is a dense union with members:
 //
-// 		Field Name (Type Code)			| Field Type
-//		----------------------------|-----------------------------
-//		string_value (0)						| utf8
-//		bool_value (1)							| bool
-//		int64_value (2)							| int64
-//		int32_bitmask (3)						| int32
-//		string_list (4)							| list<utf8>
-//		int32_to_int32_list_map (5)	| map<int32, list<int32>>
+//	Field Name (Type Code)			| Field Type
+//	----------------------------|-----------------------------
+//	string_value (0)						| utf8
+//	bool_value (1)							| bool
+//	int64_value (2)							| int64
+//	int32_bitmask (3)						| int32
+//	string_list (4)							| list<utf8>
+//	int32_to_int32_list_map (5)	| map<int32, list<int32>>
 //
 // Each metadatum is identified by an integer code. The recognized
 // codes are defined as constants. Codes [0, 10_000) are reserved
@@ -596,7 +855,7 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 	const strValTypeID arrow.UnionTypeCode = 0
 
 	if len(infoCodes) == 0 {
-		infoCodes = maps.Keys(adbcToFlightSQLInfo)
+		infoCodes = infoSupportedCodes
 	}
 
 	bldr := array.NewRecordBuilder(c.cl.Alloc, adbc.GetInfoSchema)
@@ -632,42 +891,42 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 
 	ctx = metadata.NewOutgoingContext(ctx, c.hdrs)
 	info, err := c.cl.GetSqlInfo(ctx, translated, c.timeouts)
-	if err != nil {
-		return nil, adbcFromFlightStatus(err)
-	}
+	if err == nil {
+		for _, endpoint := range info.Endpoint {
+			rdr, err := doGet(ctx, c.cl, endpoint, c.clientCache, c.timeouts)
+			if err != nil {
+				return nil, adbcFromFlightStatus(err)
+			}
 
-	for _, endpoint := range info.Endpoint {
-		rdr, err := doGet(ctx, c.cl, endpoint, c.clientCache, c.timeouts)
-		if err != nil {
-			return nil, adbcFromFlightStatus(err)
-		}
+			for rdr.Next() {
+				rec := rdr.Record()
+				field := rec.Column(0).(*array.Uint32)
+				info := rec.Column(1).(*array.DenseUnion)
 
-		for rdr.Next() {
-			rec := rdr.Record()
-			field := rec.Column(0).(*array.Uint32)
-			info := rec.Column(1).(*array.DenseUnion)
+				for i := 0; i < int(rec.NumRows()); i++ {
+					switch flightsql.SqlInfo(field.Value(i)) {
+					case flightsql.SqlInfoFlightSqlServerName:
+						infoNameBldr.Append(uint32(adbc.InfoVendorName))
+					case flightsql.SqlInfoFlightSqlServerVersion:
+						infoNameBldr.Append(uint32(adbc.InfoVendorVersion))
+					case flightsql.SqlInfoFlightSqlServerArrowVersion:
+						infoNameBldr.Append(uint32(adbc.InfoVendorArrowVersion))
+					}
 
-			for i := 0; i < int(rec.NumRows()); i++ {
-				switch flightsql.SqlInfo(field.Value(i)) {
-				case flightsql.SqlInfoFlightSqlServerName:
-					infoNameBldr.Append(uint32(adbc.InfoVendorName))
-				case flightsql.SqlInfoFlightSqlServerVersion:
-					infoNameBldr.Append(uint32(adbc.InfoVendorVersion))
-				case flightsql.SqlInfoFlightSqlServerArrowVersion:
-					infoNameBldr.Append(uint32(adbc.InfoVendorArrowVersion))
+					infoValueBldr.Append(info.TypeCode(i))
+					// we know we're only doing string fields here right now
+					v := info.Field(info.ChildID(i)).(*array.String).
+						Value(int(info.ValueOffset(i)))
+					strInfoBldr.Append(v)
 				}
+			}
 
-				infoValueBldr.Append(info.TypeCode(i))
-				// we know we're only doing string fields here right now
-				v := info.Field(info.ChildID(i)).(*array.String).
-					Value(int(info.ValueOffset(i)))
-				strInfoBldr.Append(v)
+			if rdr.Err() != nil {
+				return nil, adbcFromFlightStatus(rdr.Err())
 			}
 		}
-
-		if rdr.Err() != nil {
-			return nil, adbcFromFlightStatus(rdr.Err())
-		}
+	} else if grpcstatus.Code(err) != grpccodes.Unimplemented {
+		return nil, adbcFromFlightStatus(err)
 	}
 
 	final := bldr.NewRecord()
@@ -680,26 +939,26 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 //
 // The result is an Arrow Dataset with the following schema:
 //
-//		Field Name									| Field Type
-//		----------------------------|----------------------------
-//		catalog_name								| utf8
-//		catalog_db_schemas					| list<DB_SCHEMA_SCHEMA>
+//	Field Name									| Field Type
+//	----------------------------|----------------------------
+//	catalog_name								| utf8
+//	catalog_db_schemas					| list<DB_SCHEMA_SCHEMA>
 //
 // DB_SCHEMA_SCHEMA is a Struct with the fields:
 //
-//		Field Name									| Field Type
-//		----------------------------|----------------------------
-//		db_schema_name							| utf8
-//		db_schema_tables						|	list<TABLE_SCHEMA>
+//	Field Name									| Field Type
+//	----------------------------|----------------------------
+//	db_schema_name							| utf8
+//	db_schema_tables						|	list<TABLE_SCHEMA>
 //
 // TABLE_SCHEMA is a Struct with the fields:
 //
-//		Field Name									| Field Type
-//		----------------------------|----------------------------
-//		table_name									| utf8 not null
-//		table_type									|	utf8 not null
-//		table_columns								| list<COLUMN_SCHEMA>
-//		table_constraints						| list<CONSTRAINT_SCHEMA>
+//	Field Name									| Field Type
+//	----------------------------|----------------------------
+//	table_name									| utf8 not null
+//	table_type									|	utf8 not null
+//	table_columns								| list<COLUMN_SCHEMA>
+//	table_constraints						| list<CONSTRAINT_SCHEMA>
 //
 // COLUMN_SCHEMA is a Struct with the fields:
 //
@@ -725,20 +984,20 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 //		xdbc_is_autoincrement				| bool								| (3)
 //		xdbc_is_generatedcolumn			| bool								| (3)
 //
-// 1. The column's ordinal position in the table (starting from 1).
-// 2. Database-specific description of the column.
-// 3. Optional Value. Should be null if not supported by the driver.
-//	  xdbc_values are meant to provide JDBC/ODBC-compatible metadata
-//		in an agnostic manner.
+//	 1. The column's ordinal position in the table (starting from 1).
+//	 2. Database-specific description of the column.
+//	 3. Optional Value. Should be null if not supported by the driver.
+//	    xdbc_values are meant to provide JDBC/ODBC-compatible metadata
+//	    in an agnostic manner.
 //
 // CONSTRAINT_SCHEMA is a Struct with the fields:
 //
-//		Field Name									| Field Type					| Comments
-//		----------------------------|---------------------|---------
-//		constraint_name							| utf8								|
-//		constraint_type							| utf8 not null				| (1)
-//		constraint_column_names			| list<utf8> not null | (2)
-//		constraint_column_usage			| list<USAGE_SCHEMA>	| (3)
+//	Field Name									| Field Type					| Comments
+//	----------------------------|---------------------|---------
+//	constraint_name							| utf8								|
+//	constraint_type							| utf8 not null				| (1)
+//	constraint_column_names			| list<utf8> not null | (2)
+//	constraint_column_usage			| list<USAGE_SCHEMA>	| (3)
 //
 // 1. One of 'CHECK', 'FOREIGN KEY', 'PRIMARY KEY', or 'UNIQUE'.
 // 2. The columns on the current table that are constrained, in order.
@@ -746,12 +1005,12 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 //
 // USAGE_SCHEMA is a Struct with fields:
 //
-//		Field Name									|	Field Type
-//		----------------------------|----------------------------
-//		fk_catalog									| utf8
-//		fk_db_schema								| utf8
-//		fk_table										| utf8 not null
-//		fk_column_name							| utf8 not null
+//	Field Name									|	Field Type
+//	----------------------------|----------------------------
+//	fk_catalog									| utf8
+//	fk_db_schema								| utf8
+//	fk_table										| utf8 not null
+//	fk_column_name							| utf8 not null
 //
 // For the parameters: If nil is passed, then that parameter will not
 // be filtered by at all. If an empty string, then only objects without
@@ -764,11 +1023,11 @@ func (c *cnxn) GetInfo(ctx context.Context, infoCodes []adbc.InfoCode) (array.Re
 // earlier).
 func (c *cnxn) GetObjects(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string, tableName *string, columnName *string, tableType []string) (array.RecordReader, error) {
 	ctx = metadata.NewOutgoingContext(ctx, c.hdrs)
-	g := getObjects{ctx: ctx, depth: depth, catalog: catalog, dbSchema: dbSchema, tableName: tableName, columnName: columnName, tableType: tableType}
-	if err := g.init(c); err != nil {
+	g := internal.GetObjects{Ctx: ctx, Depth: depth, Catalog: catalog, DbSchema: dbSchema, TableName: tableName, ColumnName: columnName, TableType: tableType}
+	if err := g.Init(c.db.alloc, c.getObjectsDbSchemas, c.getObjectsTables); err != nil {
 		return nil, err
 	}
-	defer g.release()
+	defer g.Release()
 
 	// To avoid an N+1 query problem, we assume result sets here will fit in memory and build up a single response.
 	info, err := c.cl.GetCatalogs(ctx)
@@ -788,228 +1047,21 @@ func (c *cnxn) GetObjects(ctx context.Context, depth adbc.ObjectDepth, catalog *
 		for i := 0; i < arr.Len(); i++ {
 			// XXX: force copy since accessor is unsafe
 			catalogName := string([]byte(arr.Value(i)))
-			g.appendCatalog(catalogName)
+			g.AppendCatalog(catalogName)
 			foundCatalog = true
 		}
 	}
 
 	// Implementations like Dremio report no catalogs, but still have schemas
 	if !foundCatalog && depth != adbc.ObjectDepthCatalogs {
-		g.appendCatalog("")
+		g.AppendCatalog("")
 	}
 
 	if err = rdr.Err(); err != nil {
 		return nil, adbcFromFlightStatus(err)
 	}
 
-	return g.finish()
-}
-
-// Helper to store state needed for GetObjects
-type getObjects struct {
-	ctx        context.Context
-	depth      adbc.ObjectDepth
-	catalog    *string
-	dbSchema   *string
-	tableName  *string
-	columnName *string
-	tableType  []string
-
-	builder           *array.RecordBuilder
-	schemaLookup      map[string][]string
-	tableLookup       map[catalogAndSchema][]tableInfo
-	catalogPattern    *regexp.Regexp
-	columnNamePattern *regexp.Regexp
-
-	catalogNameBuilder           *array.StringBuilder
-	catalogDbSchemasBuilder      *array.ListBuilder
-	catalogDbSchemasItems        *array.StructBuilder
-	dbSchemaNameBuilder          *array.StringBuilder
-	dbSchemaTablesBuilder        *array.ListBuilder
-	dbSchemaTablesItems          *array.StructBuilder
-	tableNameBuilder             *array.StringBuilder
-	tableTypeBuilder             *array.StringBuilder
-	tableColumnsBuilder          *array.ListBuilder
-	tableColumnsItems            *array.StructBuilder
-	columnNameBuilder            *array.StringBuilder
-	ordinalPositionBuilder       *array.Int32Builder
-	remarksBuilder               *array.StringBuilder
-	xdbcDataTypeBuilder          *array.Int16Builder
-	xdbcTypeNameBuilder          *array.StringBuilder
-	xdbcColumnSizeBuilder        *array.Int32Builder
-	xdbcDecimalDigitsBuilder     *array.Int16Builder
-	xdbcNumPrecRadixBuilder      *array.Int16Builder
-	xdbcNullableBuilder          *array.Int16Builder
-	xdbcColumnDefBuilder         *array.StringBuilder
-	xdbcSqlDataTypeBuilder       *array.Int16Builder
-	xdbcDatetimeSubBuilder       *array.Int16Builder
-	xdbcCharOctetLengthBuilder   *array.Int32Builder
-	xdbcIsNullableBuilder        *array.StringBuilder
-	xdbcScopeCatalogBuilder      *array.StringBuilder
-	xdbcScopeSchemaBuilder       *array.StringBuilder
-	xdbcScopeTableBuilder        *array.StringBuilder
-	xdbcIsAutoincrementBuilder   *array.BooleanBuilder
-	xdbcIsGeneratedcolumnBuilder *array.BooleanBuilder
-	tableConstraintsBuilder      *array.ListBuilder
-}
-
-func (g *getObjects) init(c *cnxn) error {
-	if catalogToDbSchemas, err := c.getObjectsDbSchemas(g.ctx, g.depth, g.catalog, g.dbSchema); err != nil {
-		return err
-	} else {
-		g.schemaLookup = catalogToDbSchemas
-	}
-
-	if tableLookup, err := c.getObjectsTables(g.ctx, g.depth, g.catalog, g.dbSchema, g.tableName, g.columnName, g.tableType); err != nil {
-		return err
-	} else {
-		g.tableLookup = tableLookup
-	}
-
-	if catalogPattern, err := patternToRegexp(g.catalog); err != nil {
-		return adbc.Error{
-			Msg:  err.Error(),
-			Code: adbc.StatusInvalidArgument,
-		}
-	} else {
-		g.catalogPattern = catalogPattern
-	}
-	if columnNamePattern, err := patternToRegexp(g.columnName); err != nil {
-		return adbc.Error{
-			Msg:  err.Error(),
-			Code: adbc.StatusInvalidArgument,
-		}
-	} else {
-		g.columnNamePattern = columnNamePattern
-	}
-
-	g.builder = array.NewRecordBuilder(c.db.alloc, adbc.GetObjectsSchema)
-	g.catalogNameBuilder = g.builder.Field(0).(*array.StringBuilder)
-	g.catalogDbSchemasBuilder = g.builder.Field(1).(*array.ListBuilder)
-	g.catalogDbSchemasItems = g.catalogDbSchemasBuilder.ValueBuilder().(*array.StructBuilder)
-	g.dbSchemaNameBuilder = g.catalogDbSchemasItems.FieldBuilder(0).(*array.StringBuilder)
-	g.dbSchemaTablesBuilder = g.catalogDbSchemasItems.FieldBuilder(1).(*array.ListBuilder)
-	g.dbSchemaTablesItems = g.dbSchemaTablesBuilder.ValueBuilder().(*array.StructBuilder)
-	g.tableNameBuilder = g.dbSchemaTablesItems.FieldBuilder(0).(*array.StringBuilder)
-	g.tableTypeBuilder = g.dbSchemaTablesItems.FieldBuilder(1).(*array.StringBuilder)
-	g.tableColumnsBuilder = g.dbSchemaTablesItems.FieldBuilder(2).(*array.ListBuilder)
-	g.tableColumnsItems = g.tableColumnsBuilder.ValueBuilder().(*array.StructBuilder)
-	g.columnNameBuilder = g.tableColumnsItems.FieldBuilder(0).(*array.StringBuilder)
-	g.ordinalPositionBuilder = g.tableColumnsItems.FieldBuilder(1).(*array.Int32Builder)
-	g.remarksBuilder = g.tableColumnsItems.FieldBuilder(2).(*array.StringBuilder)
-	g.xdbcDataTypeBuilder = g.tableColumnsItems.FieldBuilder(3).(*array.Int16Builder)
-	g.xdbcTypeNameBuilder = g.tableColumnsItems.FieldBuilder(4).(*array.StringBuilder)
-	g.xdbcColumnSizeBuilder = g.tableColumnsItems.FieldBuilder(5).(*array.Int32Builder)
-	g.xdbcDecimalDigitsBuilder = g.tableColumnsItems.FieldBuilder(6).(*array.Int16Builder)
-	g.xdbcNumPrecRadixBuilder = g.tableColumnsItems.FieldBuilder(7).(*array.Int16Builder)
-	g.xdbcNullableBuilder = g.tableColumnsItems.FieldBuilder(8).(*array.Int16Builder)
-	g.xdbcColumnDefBuilder = g.tableColumnsItems.FieldBuilder(9).(*array.StringBuilder)
-	g.xdbcSqlDataTypeBuilder = g.tableColumnsItems.FieldBuilder(10).(*array.Int16Builder)
-	g.xdbcDatetimeSubBuilder = g.tableColumnsItems.FieldBuilder(11).(*array.Int16Builder)
-	g.xdbcCharOctetLengthBuilder = g.tableColumnsItems.FieldBuilder(12).(*array.Int32Builder)
-	g.xdbcIsNullableBuilder = g.tableColumnsItems.FieldBuilder(13).(*array.StringBuilder)
-	g.xdbcScopeCatalogBuilder = g.tableColumnsItems.FieldBuilder(14).(*array.StringBuilder)
-	g.xdbcScopeSchemaBuilder = g.tableColumnsItems.FieldBuilder(15).(*array.StringBuilder)
-	g.xdbcScopeTableBuilder = g.tableColumnsItems.FieldBuilder(16).(*array.StringBuilder)
-	g.xdbcIsAutoincrementBuilder = g.tableColumnsItems.FieldBuilder(17).(*array.BooleanBuilder)
-	g.xdbcIsGeneratedcolumnBuilder = g.tableColumnsItems.FieldBuilder(18).(*array.BooleanBuilder)
-	g.tableConstraintsBuilder = g.dbSchemaTablesItems.FieldBuilder(3).(*array.ListBuilder)
-
-	return nil
-}
-
-func (g *getObjects) release() {
-	g.builder.Release()
-}
-
-func (g *getObjects) finish() (array.RecordReader, error) {
-	record := g.builder.NewRecord()
-	defer record.Release()
-	result, err := array.NewRecordReader(g.builder.Schema(), []arrow.Record{record})
-	if err != nil {
-		return nil, adbc.Error{
-			Msg:  err.Error(),
-			Code: adbc.StatusInternal,
-		}
-	}
-	return result, nil
-}
-
-func (g *getObjects) appendCatalog(catalogName string) {
-	if g.catalogPattern != nil && !g.catalogPattern.MatchString(catalogName) {
-		return
-	}
-	g.catalogNameBuilder.Append(catalogName)
-
-	if g.depth == adbc.ObjectDepthCatalogs {
-		g.catalogDbSchemasBuilder.AppendNull()
-		return
-	}
-	g.catalogDbSchemasBuilder.Append(true)
-
-	for _, dbSchemaName := range g.schemaLookup[catalogName] {
-		g.appendDbSchema(catalogName, dbSchemaName)
-	}
-}
-
-func (g *getObjects) appendDbSchema(catalogName, dbSchemaName string) {
-	g.dbSchemaNameBuilder.Append(dbSchemaName)
-	g.catalogDbSchemasItems.Append(true)
-
-	if g.depth == adbc.ObjectDepthDBSchemas {
-		g.dbSchemaTablesBuilder.AppendNull()
-		return
-	}
-	g.dbSchemaTablesBuilder.Append(true)
-
-	for _, tableInfo := range g.tableLookup[catalogAndSchema{
-		catalog: catalogName,
-		schema:  dbSchemaName,
-	}] {
-		g.appendTableInfo(tableInfo)
-	}
-}
-
-func (g *getObjects) appendTableInfo(tableInfo tableInfo) {
-	g.tableNameBuilder.Append(tableInfo.name)
-	g.tableTypeBuilder.Append(tableInfo.tableType)
-	g.dbSchemaTablesItems.Append(true)
-
-	if g.depth == adbc.ObjectDepthTables {
-		g.tableColumnsBuilder.AppendNull()
-		g.tableConstraintsBuilder.AppendNull()
-		return
-	}
-	g.tableColumnsBuilder.Append(true)
-	// TODO: unimplemented for now
-	g.tableConstraintsBuilder.Append(true)
-
-	for colIndex, column := range tableInfo.schema.Fields() {
-		if g.columnNamePattern != nil && !g.columnNamePattern.MatchString(column.Name) {
-			continue
-		}
-		g.columnNameBuilder.Append(column.Name)
-		g.ordinalPositionBuilder.Append(int32(colIndex + 1))
-		g.remarksBuilder.AppendNull()
-		g.xdbcDataTypeBuilder.AppendNull()
-		g.xdbcTypeNameBuilder.AppendNull()
-		g.xdbcColumnSizeBuilder.AppendNull()
-		g.xdbcDecimalDigitsBuilder.AppendNull()
-		g.xdbcNumPrecRadixBuilder.AppendNull()
-		g.xdbcNullableBuilder.AppendNull()
-		g.xdbcColumnDefBuilder.AppendNull()
-		g.xdbcSqlDataTypeBuilder.AppendNull()
-		g.xdbcDatetimeSubBuilder.AppendNull()
-		g.xdbcCharOctetLengthBuilder.AppendNull()
-		g.xdbcIsNullableBuilder.AppendNull()
-		g.xdbcScopeCatalogBuilder.AppendNull()
-		g.xdbcScopeSchemaBuilder.AppendNull()
-		g.xdbcScopeTableBuilder.AppendNull()
-		g.xdbcIsAutoincrementBuilder.AppendNull()
-		g.xdbcIsGeneratedcolumnBuilder.AppendNull()
-
-		g.tableColumnsItems.Append(true)
-	}
+	return g.Finish()
 }
 
 // Helper function to read and validate a metadata stream
@@ -1030,38 +1082,6 @@ func (c *cnxn) readInfo(ctx context.Context, expectedSchema *arrow.Schema, info
 	return rdr, nil
 }
 
-// Helper function that compiles a SQL-style pattern (%, _) to a regex
-func patternToRegexp(pattern *string) (*regexp.Regexp, error) {
-	if pattern == nil {
-		return nil, nil
-	}
-
-	var builder strings.Builder
-	if _, err := builder.WriteString("^"); err != nil {
-		return nil, err
-	}
-	for _, c := range *pattern {
-		switch {
-		case c == rune('_'):
-			if _, err := builder.WriteString("."); err != nil {
-				return nil, err
-			}
-		case c == rune('%'):
-			if _, err := builder.WriteString(".*"); err != nil {
-				return nil, err
-			}
-		default:
-			if _, err := builder.WriteString(regexp.QuoteMeta(string([]rune{c}))); err != nil {
-				return nil, err
-			}
-		}
-	}
-	if _, err := builder.WriteString("$"); err != nil {
-		return nil, err
-	}
-	return regexp.Compile(builder.String())
-}
-
 // Helper function to build up a map of catalogs to DB schemas
 func (c *cnxn) getObjectsDbSchemas(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string) (result map[string][]string, err error) {
 	if depth == adbc.ObjectDepthCatalogs {
@@ -1102,24 +1122,16 @@ func (c *cnxn) getObjectsDbSchemas(ctx context.Context, depth adbc.ObjectDepth,
 	return
 }
 
-type catalogAndSchema struct {
-	catalog, schema string
-}
-
-type tableInfo struct {
-	name, tableType string
-	schema          *arrow.Schema
-}
-
-func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string, tableName *string, columnName *string, tableType []string) (result map[catalogAndSchema][]tableInfo, err error) {
+func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, catalog *string, dbSchema *string, tableName *string, columnName *string, tableType []string) (result internal.SchemaToTableInfo, err error) {
 	if depth == adbc.ObjectDepthCatalogs || depth == adbc.ObjectDepthDBSchemas {
 		return
 	}
-	result = make(map[catalogAndSchema][]tableInfo)
+	result = make(map[internal.CatalogAndSchema][]internal.TableInfo)
 
 	// Pre-populate the map of which schemas are in which catalogs
 	includeSchema := depth == adbc.ObjectDepthAll || depth == adbc.ObjectDepthColumns
 	info, err := c.cl.GetTables(ctx, &flightsql.GetTablesOpts{
+		DbSchemaFilterPattern:  dbSchema,
 		TableNameFilterPattern: tableName,
 		TableTypes:             tableType,
 		IncludeSchema:          includeSchema,
@@ -1155,9 +1167,9 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 			if !dbSchema.IsNull(i) {
 				dbSchemaName = string([]byte(dbSchema.Value(i)))
 			}
-			key := catalogAndSchema{
-				catalog: catalogName,
-				schema:  dbSchemaName,
+			key := internal.CatalogAndSchema{
+				Catalog: catalogName,
+				Schema:  dbSchemaName,
 			}
 
 			var schema *arrow.Schema
@@ -1173,10 +1185,10 @@ func (c *cnxn) getObjectsTables(ctx context.Context, depth adbc.ObjectDepth, cat
 				reader.Release()
 			}
 
-			result[key] = append(result[key], tableInfo{
-				name:      string([]byte(tableName.Value(i))),
-				tableType: string([]byte(tableType.Value(i))),
-				schema:    schema,
+			result[key] = append(result[key], internal.TableInfo{
+				Name:      string([]byte(tableName.Value(i))),
+				TableType: string([]byte(tableType.Value(i))),
+				Schema:    schema,
 			})
 		}
 	}
@@ -1243,10 +1255,9 @@ func (c *cnxn) GetTableSchema(ctx context.Context, catalog *string, dbSchema *st
 //
 // The result is an arrow dataset with the following schema:
 //
-//		Field Name			| Field Type
-//		----------------|--------------
-//		table_type			| utf8 not null
-//
+//	Field Name			| Field Type
+//	----------------|--------------
+//	table_type			| utf8 not null
 func (c *cnxn) GetTableTypes(ctx context.Context) (array.RecordReader, error) {
 	ctx = metadata.NewOutgoingContext(ctx, c.hdrs)
 	info, err := c.cl.GetTableTypes(ctx, c.timeouts)
@@ -1263,10 +1274,29 @@ func (c *cnxn) GetTableTypes(ctx context.Context) (array.RecordReader, error) {
 // Behavior is undefined if this is mixed with SQL transaction statements.
 // When not supported, the convention is that it should act as if autocommit
 // is enabled and return INVALID_STATE errors.
-func (c *cnxn) Commit(_ context.Context) error {
-	return adbc.Error{
-		Msg:  "[Flight SQL] Transaction methods are not implemented yet",
-		Code: adbc.StatusInvalidState}
+func (c *cnxn) Commit(ctx context.Context) error {
+	if c.txn == nil {
+		return adbc.Error{
+			Msg:  "[Flight SQL] Cannot commit when autocommit is enabled",
+			Code: adbc.StatusInvalidState,
+		}
+	}
+
+	if !c.supportInfo.transactions {
+		return errNoTransactionSupport
+	}
+
+	ctx = metadata.NewOutgoingContext(ctx, c.hdrs)
+	err := c.txn.Commit(ctx, c.timeouts)
+	if err != nil {
+		return adbcFromFlightStatus(err)
+	}
+
+	c.txn, err = c.cl.BeginTransaction(ctx, c.timeouts)
+	if err != nil {
+		return adbcFromFlightStatus(err)
+	}
+	return nil
 }
 
 // Rollback rolls back any pending transactions. Only used if autocommit
@@ -1275,24 +1305,91 @@ func (c *cnxn) Commit(_ context.Context) error {
 // Behavior is undefined if this is mixed with SQL transaction statements.
 // When not supported, the convention is that it should act as if autocommit
 // is enabled and return INVALID_STATE errors.
-func (c *cnxn) Rollback(_ context.Context) error {
-	return adbc.Error{
-		Msg:  "[Flight SQL] Transaction methods are not implemented yet",
-		Code: adbc.StatusInvalidState}
+func (c *cnxn) Rollback(ctx context.Context) error {
+	if c.txn == nil {
+		return adbc.Error{
+			Msg:  "[Flight SQL] Cannot rollback when autocommit is enabled",
+			Code: adbc.StatusInvalidState,
+		}
+	}
+
+	if !c.supportInfo.transactions {
+		return errNoTransactionSupport
+	}
+
+	ctx = metadata.NewOutgoingContext(ctx, c.hdrs)
+	err := c.txn.Rollback(ctx, c.timeouts)
+	if err != nil {
+		return adbcFromFlightStatus(err)
+	}
+
+	c.txn, err = c.cl.BeginTransaction(ctx, c.timeouts)
+	if err != nil {
+		return adbcFromFlightStatus(err)
+	}
+	return nil
 }
 
 // NewStatement initializes a new statement object tied to this connection
 func (c *cnxn) NewStatement() (adbc.Statement, error) {
 	return &statement{
 		alloc:       c.db.alloc,
-		cl:          c.cl,
 		clientCache: c.clientCache,
 		hdrs:        c.hdrs.Copy(),
 		queueSize:   5,
 		timeouts:    c.timeouts,
+		cnxn:        c,
 	}, nil
 }
 
+func (c *cnxn) execute(ctx context.Context, query string, opts ...grpc.CallOption) (*flight.FlightInfo, error) {
+	if c.txn != nil {
+		return c.txn.Execute(ctx, query, opts...)
+	}
+
+	return c.cl.Execute(ctx, query, opts...)
+}
+
+func (c *cnxn) executeSubstrait(ctx context.Context, plan flightsql.SubstraitPlan, opts ...grpc.CallOption) (*flight.FlightInfo, error) {
+	if c.txn != nil {
+		return c.txn.ExecuteSubstrait(ctx, plan, opts...)
+	}
+
+	return c.cl.ExecuteSubstrait(ctx, plan, opts...)
+}
+
+func (c *cnxn) executeUpdate(ctx context.Context, query string, opts ...grpc.CallOption) (n int64, err error) {
+	if c.txn != nil {
+		return c.txn.ExecuteUpdate(ctx, query, opts...)
+	}
+
+	return c.cl.ExecuteUpdate(ctx, query, opts...)
+}
+
+func (c *cnxn) executeSubstraitUpdate(ctx context.Context, plan flightsql.SubstraitPlan, opts ...grpc.CallOption) (n int64, err error) {
+	if c.txn != nil {
+		return c.txn.ExecuteSubstraitUpdate(ctx, plan, opts...)
+	}
+
+	return c.cl.ExecuteSubstraitUpdate(ctx, plan, opts...)
+}
+
+func (c *cnxn) prepare(ctx context.Context, query string, opts ...grpc.CallOption) (*flightsql.PreparedStatement, error) {
+	if c.txn != nil {
+		return c.txn.Prepare(ctx, query, opts...)
+	}
+
+	return c.cl.Prepare(ctx, query, opts...)
+}
+
+func (c *cnxn) prepareSubstrait(ctx context.Context, plan flightsql.SubstraitPlan, opts ...grpc.CallOption) (*flightsql.PreparedStatement, error) {
+	if c.txn != nil {
+		return c.txn.PrepareSubstrait(ctx, plan, opts...)
+	}
+
+	return c.cl.PrepareSubstrait(ctx, plan, opts...)
+}
+
 // Close closes this connection and releases any associated resources.
 func (c *cnxn) Close() error {
 	if c.cl == nil {
