@@ -15,6 +15,8 @@
 // specific language governing permissions and limitations
 // under the License.
 
+// Tests that use the SQLite server example.
+
 package flightsql_test
 
 import (
@@ -25,12 +27,14 @@ import (
 	"crypto/tls"
 	"crypto/x509"
 	"crypto/x509/pkix"
+	"database/sql"
 	"encoding/pem"
 	"fmt"
 	"io"
 	"math/big"
 	"net"
 	"os"
+	"path/filepath"
 	"strings"
 	"testing"
 	"time"
@@ -38,12 +42,12 @@ import (
 	"github.com/apache/arrow-adbc/go/adbc"
 	driver "github.com/apache/arrow-adbc/go/adbc/driver/flightsql"
 	"github.com/apache/arrow-adbc/go/adbc/validation"
-	"github.com/apache/arrow/go/v12/arrow"
-	"github.com/apache/arrow/go/v12/arrow/array"
-	"github.com/apache/arrow/go/v12/arrow/flight"
-	"github.com/apache/arrow/go/v12/arrow/flight/flightsql"
-	"github.com/apache/arrow/go/v12/arrow/flight/flightsql/example"
-	"github.com/apache/arrow/go/v12/arrow/memory"
+	"github.com/apache/arrow/go/v13/arrow"
+	"github.com/apache/arrow/go/v13/arrow/array"
+	"github.com/apache/arrow/go/v13/arrow/flight"
+	"github.com/apache/arrow/go/v13/arrow/flight/flightsql"
+	"github.com/apache/arrow/go/v13/arrow/flight/flightsql/example"
+	"github.com/apache/arrow/go/v13/arrow/memory"
 	"github.com/stretchr/testify/require"
 	"github.com/stretchr/testify/suite"
 	"google.golang.org/grpc"
@@ -70,6 +74,7 @@ type FlightSQLQuirks struct {
 	s      flight.Server
 	middle HeaderServerMiddleware
 	opts   []grpc.ServerOption
+	db     *sql.DB
 
 	done chan bool
 	mem  *memory.CheckedAllocator
@@ -78,17 +83,18 @@ type FlightSQLQuirks struct {
 func (s *FlightSQLQuirks) SetupDriver(t *testing.T) adbc.Driver {
 	s.middle.recordedHeaders = make(metadata.MD)
 
-	s.done = make(chan bool)
 	var err error
 	s.mem = memory.NewCheckedAllocator(memory.DefaultAllocator)
 	s.s = flight.NewServerWithMiddleware([]flight.ServerMiddleware{flight.CreateServerMiddleware(&s.middle)}, s.opts...)
-	s.srv, err = example.NewSQLiteFlightSQLServer()
+	require.NoError(t, err)
+	s.srv, err = example.NewSQLiteFlightSQLServer(s.db)
 	require.NoError(t, err)
 	s.srv.Alloc = s.mem
 
 	s.s.RegisterFlightService(flightsql.NewFlightServer(s.srv))
 	require.NoError(t, s.s.Init("localhost:0"))
 	s.s.SetShutdownOnSignals(os.Interrupt, os.Kill)
+	s.done = make(chan bool)
 	go func() {
 		defer close(s.done)
 		_ = s.s.Serve()
@@ -98,10 +104,15 @@ func (s *FlightSQLQuirks) SetupDriver(t *testing.T) adbc.Driver {
 }
 
 func (s *FlightSQLQuirks) TearDownDriver(t *testing.T, _ adbc.Driver) {
+	if s.done == nil {
+		return
+	}
+
 	s.s.Shutdown()
 	<-s.done
 	s.srv = nil
 	s.mem.AssertSize(t, 0)
+	s.done = nil
 }
 
 func (s *FlightSQLQuirks) DatabaseOptions() map[string]string {
@@ -129,6 +140,8 @@ type srvQuery string
 
 func (s srvQuery) GetQuery() string { return string(s) }
 
+func (s srvQuery) GetTransactionId() []byte { return nil }
+
 func writeTo(arr arrow.Array, idx int, w io.Writer) {
 	switch arr := arr.(type) {
 	case *array.Int8:
@@ -201,13 +214,29 @@ func (s *FlightSQLQuirks) CreateSampleTable(tableName string, r arrow.Record) er
 	return nil
 }
 
+func (s *FlightSQLQuirks) DropTable(cnxn adbc.Connection, tblname string) error {
+	stmt, err := cnxn.NewStatement()
+	if err != nil {
+		return err
+	}
+	defer stmt.Close()
+
+	if err = stmt.SetSqlQuery(`DROP TABLE IF EXISTS ` + tblname); err != nil {
+		return err
+	}
+
+	_, err = stmt.ExecuteUpdate(context.Background())
+	return err
+}
+
 func (s *FlightSQLQuirks) Alloc() memory.Allocator               { return s.mem }
 func (s *FlightSQLQuirks) BindParameter(_ int) string            { return "?" }
 func (s *FlightSQLQuirks) SupportsConcurrentStatements() bool    { return true }
 func (s *FlightSQLQuirks) SupportsPartitionedData() bool         { return true }
-func (s *FlightSQLQuirks) SupportsTransactions() bool            { return false }
+func (s *FlightSQLQuirks) SupportsTransactions() bool            { return true }
 func (s *FlightSQLQuirks) SupportsGetParameterSchema() bool      { return false }
 func (s *FlightSQLQuirks) SupportsDynamicParameterBinding() bool { return true }
+func (s *FlightSQLQuirks) SupportsBulkIngest() bool              { return false }
 func (s *FlightSQLQuirks) GetMetadata(code adbc.InfoCode) interface{} {
 	switch code {
 	case adbc.InfoDriverName:
@@ -215,22 +244,43 @@ func (s *FlightSQLQuirks) GetMetadata(code adbc.InfoCode) interface{} {
 	// runtime/debug.ReadBuildInfo doesn't currently work for tests
 	// github.com/golang/go/issues/33976
 	case adbc.InfoDriverVersion:
-		return ""
+		return "(unknown or development build)"
 	case adbc.InfoDriverArrowVersion:
-		return ""
+		return "(unknown or development build)"
 	case adbc.InfoVendorName:
 		return "db_name"
 	case adbc.InfoVendorVersion:
 		return "sqlite 3"
 	case adbc.InfoVendorArrowVersion:
-		return "12.0.0-SNAPSHOT"
+		return "13.0.0-SNAPSHOT"
 	}
 
 	return nil
 }
 
+func (s *FlightSQLQuirks) SampleTableSchemaMetadata(tblName string, dt arrow.DataType) arrow.Metadata {
+	switch dt.ID() {
+	case arrow.STRING:
+		return arrow.MetadataFrom(map[string]string{
+			flightsql.ScaleKey: "15", flightsql.IsReadOnlyKey: "0", flightsql.IsAutoIncrementKey: "0",
+			flightsql.TableNameKey: tblName,
+		})
+	default:
+		return arrow.MetadataFrom(map[string]string{
+			flightsql.ScaleKey: "15", flightsql.IsReadOnlyKey: "0", flightsql.IsAutoIncrementKey: "0",
+			flightsql.TableNameKey: tblName, flightsql.PrecisionKey: "10",
+		})
+	}
+}
+
+func (s *FlightSQLQuirks) DBSchema() string { return "" }
+
 func TestADBCFlightSQL(t *testing.T) {
-	q := &FlightSQLQuirks{}
+	db, err := example.CreateDB()
+	require.NoError(t, err)
+	defer db.Close()
+
+	q := &FlightSQLQuirks{db: db}
 	suite.Run(t, &validation.DatabaseTests{Quirks: q})
 	suite.Run(t, &validation.ConnectionTests{Quirks: q})
 	suite.Run(t, &validation.StatementTests{Quirks: q})
@@ -240,8 +290,9 @@ func TestADBCFlightSQL(t *testing.T) {
 	suite.Run(t, &OptionTests{Quirks: q})
 	suite.Run(t, &PartitionTests{Quirks: q})
 	suite.Run(t, &StatementTests{Quirks: q})
-	suite.Run(t, &TimeoutTestSuite{})
-	suite.Run(t, &TLSTests{Quirks: &FlightSQLQuirks{}})
+	suite.Run(t, &TLSTests{Quirks: &FlightSQLQuirks{db: db}})
+	suite.Run(t, &ConnectionTests{})
+	suite.Run(t, &DomainSocketTests{db: db})
 }
 
 // Driver-specific tests
@@ -494,18 +545,33 @@ func (suite *StatementTests) TearDownTest() {
 	suite.Driver = nil
 }
 
+func (suite *StatementTests) TestDuplicateColumnNames() {
+	suite.NoError(suite.Stmt.SetSqlQuery("SELECT 1 as a, 2 as a"))
+	reader, _, err := suite.Stmt.ExecuteQuery(suite.ctx)
+	suite.NoError(err)
+	defer reader.Release()
+
+	suite.Len(reader.Schema().Fields(), 2)
+	suite.Equal(reader.Schema().Field(0).Name, "a")
+	suite.Equal(reader.Schema().Field(1).Name, "a")
+
+	for reader.Next() {
+	}
+	suite.NoError(reader.Err())
+}
+
 func (suite *StatementTests) TestQueueSizeOption() {
 	var err error
-	option := "adbc.flight.sql.rpc.queue_size"
+	option := "adbc.rpc.result_queue_size"
 
 	err = suite.Stmt.SetOption(option, "")
-	suite.Require().ErrorContains(err, "Invalid value for statement option 'adbc.flight.sql.rpc.queue_size': '' is not a positive integer")
+	suite.Require().ErrorContains(err, "Invalid value for statement option '"+option+"': '' is not a positive integer")
 
 	err = suite.Stmt.SetOption(option, "foo")
-	suite.Require().ErrorContains(err, "Invalid value for statement option 'adbc.flight.sql.rpc.queue_size': 'foo' is not a positive integer")
+	suite.Require().ErrorContains(err, "Invalid value for statement option '"+option+"': 'foo' is not a positive integer")
 
 	err = suite.Stmt.SetOption(option, "-1")
-	suite.Require().ErrorContains(err, "Invalid value for statement option 'adbc.flight.sql.rpc.queue_size': '-1' is not a positive integer")
+	suite.Require().ErrorContains(err, "Invalid value for statement option '"+option+"': '-1' is not a positive integer")
 
 	err = suite.Stmt.SetOption(option, "1")
 	suite.Require().NoError(err)
@@ -516,7 +582,7 @@ func (suite *StatementTests) TestSubstrait() {
 	suite.Require().NoError(err)
 
 	_, _, err = suite.Stmt.ExecuteQuery(context.Background())
-	suite.Require().ErrorContains(err, "Substrait is not yet implemented")
+	suite.Require().ErrorContains(err, "not implemented")
 	var adbcError adbc.Error
 	suite.ErrorAs(err, &adbcError)
 	suite.Equal(adbc.StatusNotImplemented, adbcError.Code)
@@ -572,6 +638,25 @@ func (suite *HeaderTests) TestDatabaseOptAuthorization() {
 	suite.Contains(suite.Quirks.middle.recordedHeaders.Get("authorization"), "auth-header-token")
 }
 
+func (suite *HeaderTests) TestUserAgent() {
+	stmt, err := suite.Cnxn.NewStatement()
+	suite.Require().NoError(err)
+	defer func() {
+		suite.Require().NoError(stmt.Close())
+	}()
+
+	suite.Require().NoError(stmt.SetSqlQuery("timeout"))
+	_, _, err = stmt.ExecuteQuery(suite.ctx)
+	suite.Error(err)
+
+	userAgents := suite.Quirks.middle.recordedHeaders.Get("user-agent")
+	suite.NotEmpty(userAgents)
+	for _, agent := range userAgents {
+		suite.Contains(agent, "ADBC Flight SQL Driver")
+		suite.Contains(agent, "grpc-go")
+	}
+}
+
 func (suite *HeaderTests) TestConnection() {
 	// can't change authorization header on connection, you have to set it
 	// as an option on the database object when creating the connection.
@@ -655,172 +740,21 @@ func (suite *HeaderTests) TestCombined() {
 	suite.Contains(suite.Quirks.middle.recordedHeaders.Get("x-header-three"), "value 3")
 }
 
-type TimeoutTestServer struct {
-	flightsql.BaseServer
-}
+func (suite *HeaderTests) TestPrepared() {
+	stmt, err := suite.Cnxn.NewStatement()
+	suite.Require().NoError(err)
 
-func (ts *TimeoutTestServer) DoGetStatement(ctx context.Context, _ flightsql.StatementQueryTicket) (*arrow.Schema, <-chan flight.StreamChunk, error) {
-	// wait till the context is cancelled
-	<-ctx.Done()
-	return nil, nil, arrow.ErrNotImplemented
-}
+	suite.Require().NoError(stmt.SetSqlQuery("timeout"))
 
-func (ts *TimeoutTestServer) DoPutCommandStatementUpdate(ctx context.Context, cmd flightsql.StatementUpdate) (int64, error) {
-	if cmd.GetQuery() == "timeout" {
-		<-ctx.Done()
-	}
-	return 0, arrow.ErrNotImplemented
-}
+	suite.Require().NoError(suite.Cnxn.(adbc.PostInitOptions).
+		SetOption("adbc.flight.sql.rpc.call_header.x-header-one", "value 1"))
+	suite.Require().NoError(stmt.Prepare(suite.ctx))
 
-func (ts *TimeoutTestServer) GetFlightInfoStatement(ctx context.Context, cmd flightsql.StatementQuery, desc *flight.FlightDescriptor) (*flight.FlightInfo, error) {
-	switch cmd.GetQuery() {
-	case "timeout":
-		<-ctx.Done()
-	case "fetch":
-		tkt, _ := flightsql.CreateStatementQueryTicket([]byte("fetch"))
-		info := &flight.FlightInfo{
-			FlightDescriptor: desc,
-			Endpoint: []*flight.FlightEndpoint{
-				{Ticket: &flight.Ticket{Ticket: tkt}},
-			},
-			TotalRecords: -1,
-			TotalBytes:   -1,
-		}
-		return info, nil
-	}
-	return nil, arrow.ErrNotImplemented
-}
+	suite.Require().NoError(stmt.SetOption("adbc.flight.sql.rpc.call_header.x-header-two", "value 2"))
+	suite.Require().NoError(stmt.Close())
 
-type TimeoutTestSuite struct {
-	suite.Suite
-
-	s    flight.Server
-	db   adbc.Database
-	cnxn adbc.Connection
-}
-
-func (ts *TimeoutTestSuite) SetupSuite() {
-	ts.s = flight.NewServerWithMiddleware(nil)
-	ts.s.RegisterFlightService(flightsql.NewFlightServer(&TimeoutTestServer{}))
-	ts.Require().NoError(ts.s.Init("localhost:0"))
-	ts.s.SetShutdownOnSignals(os.Interrupt, os.Kill)
-	go func() {
-		_ = ts.s.Serve()
-	}()
-
-	uri := "grpc+tcp://" + ts.s.Addr().String()
-	var err error
-	ts.db, err = (driver.Driver{}).NewDatabase(map[string]string{
-		"uri": uri,
-	})
-	ts.Require().NoError(err)
-}
-
-func (ts *TimeoutTestSuite) SetupTest() {
-	var err error
-	ts.cnxn, err = ts.db.Open(context.Background())
-	ts.Require().NoError(err)
-}
-
-func (ts *TimeoutTestSuite) TearDownTest() {
-	ts.Require().NoError(ts.cnxn.Close())
-}
-
-func (ts *TimeoutTestSuite) TearDownSuite() {
-	ts.db = nil
-	ts.s.Shutdown()
-}
-
-func (ts *TimeoutTestSuite) TestInvalidValues() {
-	keys := []string{
-		"adbc.flight.sql.rpc.timeout_seconds.fetch",
-		"adbc.flight.sql.rpc.timeout_seconds.query",
-		"adbc.flight.sql.rpc.timeout_seconds.update",
-	}
-	values := []string{"1.1f", "asdf", "inf", "NaN", "-1"}
-
-	for _, k := range keys {
-		for _, v := range values {
-			ts.Run("key="+k+",val="+v, func() {
-				err := ts.cnxn.(adbc.PostInitOptions).SetOption(k, v)
-				var adbcErr adbc.Error
-				ts.ErrorAs(err, &adbcErr)
-				ts.Equal(adbc.StatusInvalidArgument, adbcErr.Code)
-				ts.ErrorContains(err, "invalid timeout option value")
-			})
-		}
-	}
-}
-
-func (ts *TimeoutTestSuite) TestRemoveTimeout() {
-	keys := []string{
-		"adbc.flight.sql.rpc.timeout_seconds.fetch",
-		"adbc.flight.sql.rpc.timeout_seconds.query",
-		"adbc.flight.sql.rpc.timeout_seconds.update",
-	}
-	for _, k := range keys {
-		ts.Run(k, func() {
-			ts.NoError(ts.cnxn.(adbc.PostInitOptions).SetOption(k, "1.0"))
-			ts.NoError(ts.cnxn.(adbc.PostInitOptions).SetOption(k, "0"))
-		})
-	}
-}
-
-func (ts *TimeoutTestSuite) TestDoActionTimeout() {
-	ts.NoError(ts.cnxn.(adbc.PostInitOptions).
-		SetOption("adbc.flight.sql.rpc.timeout_seconds.update", "0.1"))
-
-	stmt, err := ts.cnxn.NewStatement()
-	ts.Require().NoError(err)
-	defer stmt.Close()
-
-	ts.Require().NoError(stmt.SetSqlQuery("fetch"))
-	var adbcErr adbc.Error
-	ts.ErrorAs(stmt.Prepare(context.Background()), &adbcErr)
-	ts.Equal(adbc.StatusCancelled, adbcErr.Code)
-}
-
-func (ts *TimeoutTestSuite) TestDoGetTimeout() {
-	ts.NoError(ts.cnxn.(adbc.PostInitOptions).
-		SetOption("adbc.flight.sql.rpc.timeout_seconds.fetch", "0.1"))
-
-	stmt, err := ts.cnxn.NewStatement()
-	ts.Require().NoError(err)
-	defer stmt.Close()
-
-	ts.Require().NoError(stmt.SetSqlQuery("fetch"))
-	var adbcErr adbc.Error
-	_, _, err = stmt.ExecuteQuery(context.Background())
-	ts.ErrorAs(err, &adbcErr)
-	ts.Equal(adbc.StatusCancelled, adbcErr.Code)
-}
-
-func (ts *TimeoutTestSuite) TestDoPutTimeout() {
-	ts.NoError(ts.cnxn.(adbc.PostInitOptions).
-		SetOption("adbc.flight.sql.rpc.timeout_seconds.update", "0.1"))
-
-	stmt, err := ts.cnxn.NewStatement()
-	ts.Require().NoError(err)
-	defer stmt.Close()
-
-	ts.Require().NoError(stmt.SetSqlQuery("timeout"))
-	_, err = stmt.ExecuteUpdate(context.Background())
-	ts.Error(err)
-}
-
-func (ts *TimeoutTestSuite) TestGetFlightInfoTimeout() {
-	ts.NoError(ts.cnxn.(adbc.PostInitOptions).
-		SetOption("adbc.flight.sql.rpc.timeout_seconds.query", "0.1"))
-
-	stmt, err := ts.cnxn.NewStatement()
-	ts.Require().NoError(err)
-	defer stmt.Close()
-
-	ts.Require().NoError(stmt.SetSqlQuery("timeout"))
-	var adbcErr adbc.Error
-	_, _, err = stmt.ExecuteQuery(context.Background())
-	ts.ErrorAs(err, &adbcErr)
-	ts.NotEqual(adbc.StatusNotImplemented, adbcErr.Code)
+	suite.Contains(suite.Quirks.middle.recordedHeaders.Get("x-header-one"), "value 1")
+	suite.Contains(suite.Quirks.middle.recordedHeaders.Get("x-header-two"), "value 2")
 }
 
 type TLSTests struct {
@@ -922,3 +856,153 @@ func (suite *TLSTests) TestInvalidOptions() {
 	_, _, err = stmt.ExecuteQuery(suite.ctx)
 	suite.Contains(err.Error(), "Unavailable")
 }
+
+type ConnectionTests struct {
+	suite.Suite
+
+	alloc   *memory.CheckedAllocator
+	server  flight.Server
+	service *flight.BaseFlightServer
+
+	Driver adbc.Driver
+	DB     adbc.Database
+	Cnxn   adbc.Connection
+	ctx    context.Context
+}
+
+func (suite *ConnectionTests) SetupSuite() {
+	suite.alloc = memory.NewCheckedAllocator(memory.DefaultAllocator)
+
+	suite.server = flight.NewServerWithMiddleware(nil)
+	suite.NoError(suite.server.Init("localhost:0"))
+	suite.service = &flight.BaseFlightServer{}
+	suite.server.RegisterFlightService(suite.service)
+
+	go func() {
+		// Explicitly ignore error
+		_ = suite.server.Serve()
+	}()
+
+	var err error
+	suite.ctx = context.Background()
+	suite.Driver = driver.Driver{Alloc: suite.alloc}
+	suite.DB, err = suite.Driver.NewDatabase(map[string]string{
+		adbc.OptionKeyURI: "grpc+tcp://" + suite.server.Addr().String(),
+	})
+	suite.Require().NoError(err)
+	suite.Cnxn, err = suite.DB.Open(suite.ctx)
+	suite.Require().NoError(err)
+}
+
+func (suite *ConnectionTests) TearDownSuite() {
+	suite.server.Shutdown()
+	suite.alloc.AssertSize(suite.T(), 0)
+}
+
+func (suite *ConnectionTests) TestGetInfo() {
+	reader, err := suite.Cnxn.GetInfo(suite.ctx, []adbc.InfoCode{})
+	// Should not error, even though server does not implement GetInfo
+	suite.Require().NoError(err)
+	defer reader.Release()
+
+	driverName := false
+	driverVersion := false
+	driverArrowVersion := false
+	for reader.Next() {
+		code := reader.Record().Column(0).(*array.Uint32)
+		values := reader.Record().Column(1).(*array.DenseUnion)
+		stringValues := values.Field(0).(*array.String)
+		for i := 0; i < int(reader.Record().NumRows()); i++ {
+			switch adbc.InfoCode(code.Value(i)) {
+			case adbc.InfoDriverName:
+				{
+					driverName = true
+					suite.Require().Equal("ADBC Flight SQL Driver - Go", stringValues.Value(int(values.ValueOffset(i))))
+				}
+			case adbc.InfoDriverVersion:
+				{
+					driverVersion = true
+					// Can't assert on value here since test won't have debug.ReadBuildInfo
+				}
+			case adbc.InfoDriverArrowVersion:
+				{
+					driverArrowVersion = true
+					// Can't assert on value here since test won't have debug.ReadBuildInfo
+				}
+			}
+		}
+	}
+	suite.Require().NoError(reader.Err())
+
+	suite.Require().True(driverName)
+	suite.Require().True(driverVersion)
+	suite.Require().True(driverArrowVersion)
+}
+
+type DomainSocketTests struct {
+	suite.Suite
+
+	alloc   *memory.CheckedAllocator
+	server  flight.Server
+	service *example.SQLiteFlightSQLServer
+	db      *sql.DB
+
+	Driver adbc.Driver
+	DB     adbc.Database
+	Cnxn   adbc.Connection
+	Stmt   adbc.Statement
+	ctx    context.Context
+}
+
+func (suite *DomainSocketTests) SetupSuite() {
+	suite.alloc = memory.NewCheckedAllocator(memory.DefaultAllocator)
+
+	tempDir, err := os.MkdirTemp("", "adbc-flight-sql-tests-*")
+	suite.NoError(err)
+	defer os.RemoveAll(tempDir)
+
+	listenSocket := filepath.Join(tempDir, "adbc.sock")
+
+	listener, err := net.Listen("unix", listenSocket)
+	suite.NoError(err)
+
+	suite.server = flight.NewServerWithMiddleware(nil)
+	suite.service, err = example.NewSQLiteFlightSQLServer(suite.db)
+	suite.NoError(err)
+	suite.server.RegisterFlightService(flightsql.NewFlightServer(suite.service))
+	suite.server.InitListener(listener)
+
+	go func() {
+		// Explicitly ignore error
+		_ = suite.server.Serve()
+	}()
+
+	suite.ctx = context.Background()
+	suite.Driver = driver.Driver{Alloc: suite.alloc}
+	suite.DB, err = suite.Driver.NewDatabase(map[string]string{
+		adbc.OptionKeyURI: "grpc+unix://" + listenSocket,
+	})
+	suite.Require().NoError(err)
+	suite.Cnxn, err = suite.DB.Open(suite.ctx)
+	suite.Require().NoError(err)
+	suite.Stmt, err = suite.Cnxn.NewStatement()
+	suite.Require().NoError(err)
+}
+
+func (suite *DomainSocketTests) TearDownSuite() {
+	suite.Require().NoError(suite.Stmt.Close())
+	suite.Require().NoError(suite.Cnxn.Close())
+	suite.server.Shutdown()
+	suite.alloc.AssertSize(suite.T(), 0)
+}
+
+func (suite *DomainSocketTests) TestSimpleQueryDomainSocket() {
+	suite.NoError(suite.Stmt.SetSqlQuery("SELECT 1"))
+	reader, _, err := suite.Stmt.ExecuteQuery(suite.ctx)
+	suite.NoError(err)
+	defer reader.Release()
+
+	for reader.Next() {
+	}
+	suite.NoError(reader.Err())
+}
