@@ -15,20 +15,6 @@
 // specific language governing permissions and limitations
 // under the License.
 
-// Package sqldriver is a wrapper around the ADBC (Arrow Database
-// Connectivity) interfaces to support the standard golang database/sql
-// package.
-//
-// This allows any ADBC driver implementation to also be used as-is
-// with the database/sql package of the standard library rather than
-// having to implement drivers for both separately.
-//
-// Registering the driver can be done by importing this and then running
-//
-// 		sql.Register("drivername", sqldriver.Driver{adbcdriver})
-//
-// EXPERIMENTAL. The ADBC interfaces are subject to change and as such
-// this wrapper is also subject to change based on that.
 package sqldriver
 
 import (
@@ -44,11 +30,11 @@ import (
 	"unsafe"
 
 	"github.com/apache/arrow-adbc/go/adbc"
-	"github.com/apache/arrow/go/v12/arrow"
-	"github.com/apache/arrow/go/v12/arrow/array"
-	"github.com/apache/arrow/go/v12/arrow/decimal128"
-	"github.com/apache/arrow/go/v12/arrow/decimal256"
-	"github.com/apache/arrow/go/v12/arrow/memory"
+	"github.com/apache/arrow/go/v13/arrow"
+	"github.com/apache/arrow/go/v13/arrow/array"
+	"github.com/apache/arrow/go/v13/arrow/decimal128"
+	"github.com/apache/arrow/go/v13/arrow/decimal256"
+	"github.com/apache/arrow/go/v13/arrow/memory"
 )
 
 func getIsolationlevel(lvl sql.IsolationLevel) adbc.OptionIsolationLevel {
@@ -267,6 +253,11 @@ func (c *conn) PrepareContext(ctx context.Context, query string) (driver.Stmt, e
 		return nil, err
 	}
 
+	if err := s.Prepare(ctx); err != nil {
+		s.Close()
+		return nil, err
+	}
+
 	paramSchema, err := s.GetParameterSchema()
 	var adbcErr adbc.Error
 	if errors.As(err, &adbcErr) {
@@ -589,6 +580,9 @@ func (r *rows) Next(dest []driver.Value) error {
 
 	for r.curRecord == nil {
 		if !r.rdr.Next() {
+			if err := r.rdr.Err(); err != nil {
+				return err
+			}
 			return io.EOF
 		}
 		r.curRecord = r.rdr.Record()
