@@ -15,11 +15,23 @@
 # specific language governing permissions and limitations
 # under the License.
 
-"""
-PEP 249 (DB-API 2.0) API wrapper for the ADBC Driver Manager.
+"""PEP 249 (DB-API 2.0) API wrapper for the ADBC Driver Manager.
+
+Resource Management
+===================
+
+You must ``close()`` Connection and Cursor objects, or else driver
+resources may be leaked.  A ``__del__`` is implemented as a fallback,
+but Python does not guarantee the timing of when this is called.  For
+development, ``__del__`` will raise a ResourceWarning when running
+under pytest, or when the environment variable
+``_ADBC_DRIVER_MANAGER_WARN_UNCLOSED_RESOURCE`` is set to ``1``.
+
 """
 
+import abc
 import datetime
+import os
 import threading
 import time
 import typing
@@ -34,9 +46,8 @@ except ImportError as e:
 from . import _lib
 
 if typing.TYPE_CHECKING:
-    from typing import Self
-
     import pandas
+    from typing_extensions import Self
 
 # ----------------------------------------------------------
 # Globals
@@ -86,12 +97,12 @@ def DateFromTicks(ticks: int) -> Date:
     return Date(*time.localtime(ticks)[:3])
 
 
-def TimeFromTicks(ticks: int) -> Date:
+def TimeFromTicks(ticks: int) -> Time:
     """Construct a time value from a count of seconds."""
     return Time(*time.localtime(ticks)[3:6])
 
 
-def TimestampFromTicks(ticks: int) -> Date:
+def TimestampFromTicks(ticks: int) -> Timestamp:
     """Construct a timestamp value from a count of seconds."""
     return Timestamp(*time.localtime(ticks)[:6])
 
@@ -146,9 +157,10 @@ STRING = _TypeSet([pyarrow.string().id, pyarrow.large_string().id])
 def connect(
     *,
     driver: str,
-    entrypoint: str = None,
+    entrypoint: Optional[str] = None,
     db_kwargs: Optional[Dict[str, str]] = None,
     conn_kwargs: Optional[Dict[str, str]] = None,
+    autocommit=False,
 ) -> "Connection":
     """
     Connect to a database via ADBC.
@@ -169,6 +181,10 @@ def connect(
     conn_kwargs
         Key-value parameters to pass to the driver to initialize the
         connection.
+    autocommit
+        Whether to enable autocommit.  For compliance with DB-API,
+        this is disabled by default.  A warning will be emitted if it
+        cannot be disabled.
     """
     db = None
     conn = None
@@ -183,7 +199,7 @@ def connect(
     try:
         db = _lib.AdbcDatabase(**db_kwargs)
         conn = _lib.AdbcConnection(db, **conn_kwargs)
-        return Connection(db, conn, conn_kwargs)
+        return Connection(db, conn, conn_kwargs, autocommit=autocommit)
     except Exception:
         if conn:
             conn.close()
@@ -196,7 +212,7 @@ def connect(
 # Classes
 
 
-class _Closeable:
+class _Closeable(abc.ABC):
     """Base class providing context manager interface."""
 
     def __enter__(self) -> "Self":
@@ -205,6 +221,10 @@ class _Closeable:
     def __exit__(self, exc_type, exc_val, exc_tb) -> None:
         self.close()
 
+    @abc.abstractmethod
+    def close(self) -> None:
+        ...
+
 
 class _SharedDatabase(_Closeable):
     """A holder for a shared AdbcDatabase."""
@@ -256,7 +276,10 @@ class Connection(_Closeable):
         db: Union[_lib.AdbcDatabase, _SharedDatabase],
         conn: _lib.AdbcConnection,
         conn_kwargs: Optional[Dict[str, str]] = None,
+        *,
+        autocommit=False,
     ) -> None:
+        self._closed = False
         if isinstance(db, _SharedDatabase):
             self._db = db.clone()
         else:
@@ -268,13 +291,20 @@ class Connection(_Closeable):
             self._conn.set_autocommit(False)
         except _lib.NotSupportedError:
             self._commit_supported = False
-            warnings.warn(
-                "Cannot disable autocommit; conn will not be DB-API 2.0 compliant",
-                category=Warning,
-            )
+            if not autocommit:
+                warnings.warn(
+                    "Cannot disable autocommit; conn will not be DB-API 2.0 compliant",
+                    category=Warning,
+                )
+            self._autocommit = True
         else:
+            self._autocommit = False
             self._commit_supported = True
 
+        if autocommit and self._commit_supported:
+            self._conn.set_autocommit(True)
+            self._autocommit = True
+
     def close(self) -> None:
         """
         Close the connection.
@@ -284,8 +314,12 @@ class Connection(_Closeable):
         Failure to close a connection may leak memory or database
         connections.
         """
+        if self._closed:
+            return
+
         self._conn.close()
         self._db.close()
+        self._closed = True
 
     def commit(self) -> None:
         """Explicitly commit."""
@@ -301,6 +335,11 @@ class Connection(_Closeable):
         if self._commit_supported:
             self._conn.rollback()
 
+    def __del__(self) -> None:
+        if not self._closed:
+            self.close()
+            _warn_unclosed("adbc_driver_manager.dbapi.Connection")
+
     # ------------------------------------------------------------
     # API Extensions
     # ------------------------------------------------------------
@@ -460,8 +499,12 @@ class Cursor(_Closeable):
     """
 
     def __init__(self, conn: Connection) -> None:
+        # Must be at top in case __init__ is interrupted and then __del__ is called
+        self._closed = True
         self._conn = conn
         self._stmt = _lib.AdbcStatement(conn._conn)
+        self._closed = False
+
         self._last_query: Optional[Union[str, bytes]] = None
         self._results: Optional["_RowIterator"] = None
         self._arraysize = 1
@@ -512,9 +555,14 @@ class Cursor(_Closeable):
 
     def close(self):
         """Close the cursor and free resources."""
+        if self._closed:
+            return
+
         if self._results is not None:
             self._results.close()
+            self._results = None
         self._stmt.close()
+        self._closed = True
 
     def _bind(self, parameters) -> None:
         if isinstance(parameters, pyarrow.RecordBatch):
@@ -618,7 +666,7 @@ class Cursor(_Closeable):
         self._bind(arrow_parameters)
         self._rowcount = self._stmt.execute_update()
 
-    def fetchone(self) -> tuple:
+    def fetchone(self) -> Optional[tuple]:
         """Fetch one row of the result."""
         if self._results is None:
             raise ProgrammingError(
@@ -666,6 +714,11 @@ class Cursor(_Closeable):
         """Preallocate memory for the result set (no-op)."""
         pass
 
+    def __del__(self) -> None:
+        if not self._closed:
+            self.close()
+            _warn_unclosed("adbc_driver_manager.dbapi.Cursor")
+
     def __iter__(self):
         return self
 
@@ -741,8 +794,8 @@ class Cursor(_Closeable):
         """
         Execute a query and get the partitions of a distributed result set.
 
-        Parameters
-        ----------
+        Returns
+        -------
         partitions : list of byte
             A list of partition descriptors, which can be read with
             read_partition.
@@ -757,6 +810,32 @@ class Cursor(_Closeable):
         partitions, schema, self._rowcount = self._stmt.execute_partitions()
         return partitions, pyarrow.Schema._import_from_c(schema.address)
 
+    def adbc_prepare(self, operation: Union[bytes, str]) -> Optional[pyarrow.Schema]:
+        """
+        Prepare a query without executing it.
+
+        To execute the query afterwards, call :meth:`execute` or
+        :meth:`executemany` with the same query.  This will not
+        prepare the query a second time.
+
+        Returns
+        -------
+        pyarrow.Schema or None
+            The schema of the bind parameters, or None if the schema
+            could not be determined.
+
+        Notes
+        -----
+        This is an extension and not part of the DBAPI standard.
+        """
+        self._prepare_execute(operation)
+
+        try:
+            handle = self._stmt.get_parameter_schema()
+        except NotSupportedError:
+            return None
+        return pyarrow.Schema._import_from_c(handle.address)
+
     def adbc_read_partition(self, partition: bytes) -> None:
         """
         Read a partition of a distributed result set.
@@ -766,7 +845,7 @@ class Cursor(_Closeable):
         This is an extension and not part of the DBAPI standard.
         """
         self._results = None
-        handle = self.conn._conn.read_partition(partition)
+        handle = self._conn._conn.read_partition(partition)
         self._rowcount = -1
         self._results = _RowIterator(
             pyarrow.RecordBatchReader._import_from_c(handle.address)
@@ -783,6 +862,24 @@ class Cursor(_Closeable):
         """
         return self._stmt
 
+    def executescript(self, operation: str) -> None:
+        """
+        Execute multiple statements.
+
+        If there is a pending transaction, commits first.
+
+        Notes
+        -----
+        This is an extension and not part of the DBAPI standard.
+        """
+        if not self._conn._autocommit:
+            self._conn.commit()
+
+        self._last_query = None
+        self._results = None
+        self._stmt.set_sql_query(operation)
+        self._stmt.execute_update()
+
     def fetchallarrow(self) -> pyarrow.Table:
         """
         Fetch all rows of the result as a PyArrow Table.
@@ -830,6 +927,10 @@ class Cursor(_Closeable):
         return self._results.fetch_df()
 
 
+# ----------------------------------------------------------
+# Utilities
+
+
 class _RowIterator(_Closeable):
     """Track state needed to iterate over the result set."""
 
@@ -852,16 +953,19 @@ class _RowIterator(_Closeable):
             for field in self._reader.schema
         ]
 
-    def fetchone(self):
+    def fetchone(self) -> Optional[tuple]:
         if self._current_batch is None or self._next_row >= len(self._current_batch):
             try:
-                self._current_batch = self._reader.read_next_batch()
+                while True:
+                    self._current_batch = self._reader.read_next_batch()
+                    if self._current_batch.num_rows > 0:
+                        break
                 self._next_row = 0
             except StopIteration:
                 self._current_batch = None
                 self._finished = True
 
-        if self._finished:
+        if self._finished or self._current_batch is None:
             return None
 
         row = tuple(arr[self._next_row].as_py() for arr in self._current_batch.columns)
@@ -892,3 +996,18 @@ class _RowIterator(_Closeable):
 
     def fetch_df(self):
         return self._reader.read_pandas()
+
+
+_PYTEST_ENV_VAR = "PYTEST_CURRENT_TEST"
+_ADBC_ENV_VAR = "_ADBC_DRIVER_MANAGER_WARN_UNCLOSED_RESOURCE"
+
+
+def _warn_unclosed(name):
+    if _PYTEST_ENV_VAR in os.environ or os.environ.get(_ADBC_ENV_VAR) == "1":
+        warnings.warn(
+            f"A {name} was not explicitly close()d, which may leak "
+            f"driver resources. (This warning is only emitted if "
+            f"{_PYTEST_ENV_VAR} or {_ADBC_ENV_VAR} are set.)",
+            category=ResourceWarning,
+            stacklevel=2,
+        )
