@@ -46,7 +46,7 @@ def _bind(stmt, batch):
 
 
 def test_version():
-    assert adbc_driver_manager.__version__
+    assert adbc_driver_manager.__version__  # type:ignore
 
 
 def test_database_init():
@@ -58,6 +58,60 @@ def test_database_init():
             pass
 
 
+def test_error_mapping():
+    import adbc_driver_manager._lib as _lib
+    from adbc_driver_manager import AdbcStatusCode
+
+    cases = [
+        (adbc_driver_manager.OperationalError, AdbcStatusCode.UNKNOWN),
+        (adbc_driver_manager.NotSupportedError, AdbcStatusCode.NOT_IMPLEMENTED),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.NOT_FOUND),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.ALREADY_EXISTS),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.INVALID_ARGUMENT),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.INVALID_STATE),
+        (adbc_driver_manager.DataError, AdbcStatusCode.INVALID_DATA),
+        (adbc_driver_manager.IntegrityError, AdbcStatusCode.INTEGRITY),
+        (adbc_driver_manager.InternalError, AdbcStatusCode.INTERNAL),
+        (adbc_driver_manager.OperationalError, AdbcStatusCode.IO),
+        (adbc_driver_manager.OperationalError, AdbcStatusCode.CANCELLED),
+        (adbc_driver_manager.OperationalError, AdbcStatusCode.TIMEOUT),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.UNAUTHENTICATED),
+        (adbc_driver_manager.ProgrammingError, AdbcStatusCode.UNAUTHORIZED),
+    ]
+
+    message = "Message"
+    for (klass, code) in cases:
+        with pytest.raises(klass) as exc_info:
+            _lib._test_error(code, message, vendor_code=None, sqlstate=None)
+        assert message in exc_info.value.args[0]
+        assert exc_info.value.status_code == code
+        assert exc_info.value.vendor_code is None
+        assert exc_info.value.sqlstate is None
+
+        with pytest.raises(klass) as exc_info:
+            _lib._test_error(code, message, vendor_code=42, sqlstate="X0000")
+        assert message in exc_info.value.args[0]
+        assert exc_info.value.status_code == code
+        assert exc_info.value.vendor_code == 42
+        assert exc_info.value.sqlstate == "X0000"
+
+
+@pytest.mark.sqlite
+def test_database_set_options(sqlite):
+    db, _ = sqlite
+    with pytest.raises(
+        adbc_driver_manager.NotSupportedError,
+        match="Unknown database option foo=bar",
+    ):
+        db.set_options(foo="bar")
+
+    with pytest.raises(
+        adbc_driver_manager.NotSupportedError,
+        match=r"Unknown database option foo=\(NULL\)",
+    ):
+        db.set_options(foo=None)
+
+
 @pytest.mark.sqlite
 def test_connection_get_info(sqlite):
     _, conn = sqlite
@@ -139,7 +193,30 @@ def test_connection_get_table_types(sqlite):
 
 
 @pytest.mark.sqlite
-def test_query(sqlite):
+def test_connection_read_partition(sqlite):
+    _, conn = sqlite
+    with pytest.raises(adbc_driver_manager.NotSupportedError):
+        conn.read_partition(b"")
+
+
+@pytest.mark.sqlite
+def test_connection_set_options(sqlite):
+    _, conn = sqlite
+    with pytest.raises(
+        adbc_driver_manager.NotSupportedError,
+        match="Unknown connection option foo=bar",
+    ):
+        conn.set_options(foo="bar")
+
+    with pytest.raises(
+        adbc_driver_manager.NotSupportedError,
+        match=r"Unknown connection option foo=\(NULL\)",
+    ):
+        conn.set_options(foo=None)
+
+
+@pytest.mark.sqlite
+def test_statement_query(sqlite):
     _, conn = sqlite
     with adbc_driver_manager.AdbcStatement(conn) as stmt:
         stmt.set_sql_query("SELECT 1")
@@ -149,7 +226,7 @@ def test_query(sqlite):
 
 
 @pytest.mark.sqlite
-def test_prepared(sqlite):
+def test_statement_prepared(sqlite):
     _, conn = sqlite
     with adbc_driver_manager.AdbcStatement(conn) as stmt:
         stmt.set_sql_query("SELECT ?")
@@ -162,7 +239,7 @@ def test_prepared(sqlite):
 
 
 @pytest.mark.sqlite
-def test_ingest(sqlite):
+def test_statement_ingest(sqlite):
     _, conn = sqlite
     data = pyarrow.record_batch(
         [
@@ -183,7 +260,27 @@ def test_ingest(sqlite):
 
 
 @pytest.mark.sqlite
-def test_autocommit(sqlite):
+def test_statement_adbc_prepare(sqlite):
+    _, conn = sqlite
+    with adbc_driver_manager.AdbcStatement(conn) as stmt:
+        stmt.set_sql_query("SELECT 1")
+        stmt.prepare()
+        handle = stmt.get_parameter_schema()
+        assert _import(handle) == pyarrow.schema([])
+
+        stmt.set_sql_query("SELECT 1 + ?")
+        stmt.prepare()
+        handle = stmt.get_parameter_schema()
+        assert _import(handle) == pyarrow.schema([("0", "null")])
+
+        _bind(stmt, pyarrow.record_batch([[41]], names=["0"]))
+        handle, _ = stmt.execute_query()
+        table = _import(handle).read_all()
+        assert table == pyarrow.table([[42]], names=["1 + ?"])
+
+
+@pytest.mark.sqlite
+def test_statement_autocommit(sqlite):
     _, conn = sqlite
 
     # Autocommit enabled by default
@@ -256,6 +353,24 @@ def test_autocommit(sqlite):
         assert table == pyarrow.Table.from_batches([data])
 
 
+@pytest.mark.sqlite
+def test_statement_set_options(sqlite):
+    _, conn = sqlite
+
+    with adbc_driver_manager.AdbcStatement(conn) as stmt:
+        with pytest.raises(
+            adbc_driver_manager.NotSupportedError,
+            match="Unknown statement option foo=bar",
+        ):
+            stmt.set_options(foo="bar")
+
+        with pytest.raises(
+            adbc_driver_manager.NotSupportedError,
+            match=r"Unknown statement option foo=\(NULL\)",
+        ):
+            stmt.set_options(foo=None)
+
+
 @pytest.mark.sqlite
 def test_child_tracking(sqlite):
     with adbc_driver_manager.AdbcDatabase(driver="adbc_driver_sqlite") as db:
