@@ -15,7 +15,7 @@
 # specific language governing permissions and limitations
 # under the License.
 
-import os
+from typing import Generator
 
 import pytest
 
@@ -23,15 +23,27 @@ from adbc_driver_postgresql import dbapi
 
 
 @pytest.fixture
-def postgres():
-    postgres_uri = os.environ.get("ADBC_POSTGRESQL_TEST_URI")
-    if not postgres_uri:
-        pytest.skip("Set ADBC_POSTGRESQL_TEST_URI to run tests")
+def postgres(postgres_uri: str) -> Generator[dbapi.Connection, None, None]:
     with dbapi.connect(postgres_uri) as conn:
         yield conn
 
 
-def test_query_trivial(postgres):
+def test_query_trivial(postgres: dbapi.Connection):
     with postgres.cursor() as cur:
         cur.execute("SELECT 1")
         assert cur.fetchone() == (1,)
+
+
+def test_ddl(postgres: dbapi.Connection):
+    with postgres.cursor() as cur:
+        cur.execute("DROP TABLE IF EXISTS test_ddl")
+        assert cur.fetchone() is None
+
+        cur.execute("CREATE TABLE test_ddl (ints INT)")
+        assert cur.fetchone() is None
+
+        cur.execute("INSERT INTO test_ddl VALUES (1) RETURNING ints")
+        assert cur.fetchone() == (1,)
+
+        cur.execute("SELECT * FROM test_ddl")
+        assert cur.fetchone() == (1,)
